# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.optimization import v2_submit_params, v2_retrieve_result_params
from ...types.post_response import PostResponse
from ...types.optimization.job_param import JobParam
from ...types.optimization.vehicle_param import VehicleParam
from ...types.optimization.shipment_param import ShipmentParam
from ...types.optimization.v2_retrieve_result_response import V2RetrieveResultResponse

__all__ = ["V2Resource", "AsyncV2Resource"]


class V2Resource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> V2ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return V2ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> V2ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return V2ResourceWithStreamingResponse(self)

    def retrieve_result(
        self,
        *,
        id: str,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> V2RetrieveResultResponse:
        """
        Flexible GET

        Args:
          id: The unique ID that was returned on successful submission of the Optimization
              POST request.

          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/optimization/v2/result",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "key": key,
                    },
                    v2_retrieve_result_params.V2RetrieveResultParams,
                ),
            ),
            cast_to=V2RetrieveResultResponse,
        )

    def submit(
        self,
        *,
        key: str,
        locations: v2_submit_params.Locations,
        vehicles: Iterable[VehicleParam],
        cost_matrix: Iterable[Iterable[int]] | NotGiven = NOT_GIVEN,
        depots: Iterable[v2_submit_params.Depot] | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        distance_matrix: Iterable[Iterable[int]] | NotGiven = NOT_GIVEN,
        duration_matrix: Iterable[Iterable[int]] | NotGiven = NOT_GIVEN,
        existing_solution_id: str | NotGiven = NOT_GIVEN,
        jobs: Iterable[JobParam] | NotGiven = NOT_GIVEN,
        options: v2_submit_params.Options | NotGiven = NOT_GIVEN,
        relations: Iterable[v2_submit_params.Relation] | NotGiven = NOT_GIVEN,
        shipments: Iterable[ShipmentParam] | NotGiven = NOT_GIVEN,
        solution: Iterable[v2_submit_params.Solution] | NotGiven = NOT_GIVEN,
        unassigned: v2_submit_params.Unassigned | NotGiven = NOT_GIVEN,
        zones: Iterable[v2_submit_params.Zone] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PostResponse:
        """
        Flexible POST

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          locations: The `locations` object is used to define all the locations that will be used
              during the optimization process. Read more about this attribute in the
              [Location Object](#location-object) section.

          vehicles: The `vehicles` attribute describes the characteristics and constraints of the
              vehicles that will be used for fulfilling the tasks. Read more about this
              attribute in the [Vehicle Object](#vehicle-object) section.

          cost_matrix: An array of arrays to denote the user-defined costs of traveling between each
              pair of geographic coordinates mentioned in the `location` array. The number of
              arrays should be equal to the number of coordinate points mentioned in the
              `location` array and each array should contain the same number of elements as
              well. Please note that `cost_matrix` is effective only when
              `travel_cost=customized`. Read more about this attribute in the
              [Custom Cost Matrix](#custom-cost-matrix) section.

          depots: `depots` object is used to collect the details of a depot. Depots can be used as
              a starting point and/or ending point for the routes and vehicles. They also can
              be used to fulfil pickup and delivery type`jobs` . The loads which are to be
              delivered at task locations will be picked from depots and loads picked-up from
              task locations will be delivered back to the depots. A depot can be configured
              using the following fields:

          description: Define the optimization job using any custom message. This description is
              returned as is in the response.

          distance_matrix: An array of arrays to denote the user-defined distances, in meters, for
              travelling between each pair of geographic coordinates mentioned in the
              `location` array. When this input is provided, actual distances between the
              locations will be ignored in favor of the values provided in this input for any
              distance calculations during the optimization process. The values provided here
              will also be used for cost calculations when `travel_cost` is “distance”.

              The number of arrays in the input should be equal to the number of coordinate
              points mentioned in the `location` array and each array, in turn, should contain
              the same number of elements as well.

              **Note:**

              - `duration_matrix` is mandatory when using`distance_matrix`.
              - When using `distance_matrix` route geometry will not be available in the
                optimized solution.

          duration_matrix: An array of arrays to denote the user-defined durations, in seconds, for
              travelling between each pair of geographic coordinates mentioned in the
              `location` array. When this input is provided, actual durations between the
              locations will be ignored in favor of the values provided in the matrix for any
              ETA calculations during the optimization process. The values provided in the
              matrix will also be used for cost calculations when `travel_cost` is “duration”.

              The number of arrays in the input should be equal to the number of coordinate
              points mentioned in the `location` array and each array, in turn, should contain
              the same number of elements as well.

              Please note that, unlike `distance_matrix`, `duration_matrix` can be used
              independently in following cases:

              - when `travel_cost` is “duration”
              - when `travel_cost` is “customized” and a `cost_matrix` is provided

              Also, the route geometry will not be available in the optimized solution when
              using `duration_matrix`.

          existing_solution_id: The previous optimization request id used to retrieve solution for
              reoptimization

          jobs: `jobs` object is used to collect the details of a particular job or task that
              needs to be completed as part of the optimization process. Each job can have
              either a `pickup` or `delivery` step, but not both. Read more about this
              attribute in the [Job Object](#job-object) section.

              Please note that either the `jobs` or the `shipments` attribute should be
              specified to build a valid request.

          options: It represents the set of options that can be used to configure optimization
              algorithms so that the solver provides a solution that meets the desired
              business objectives.

          relations: `relations` attribute is an array of individual relation objects. `type`
              parameter and `steps` object are mandatory when using this attribute.

              Please note:

              - The soft constraints are **not** effective when using the `relations`
                attribute.
              - In case a given relation can't be satisfied, the optimizer will flag all the
                tasks involved in that "relation" as unassigned.

              Read more about this attribute in the [Relations Object](#relations-object)
              section.

          shipments: The `shipments` object is used to collect the details of shipments that need to
              be completed as part of the optimization process.

              Each shipment should have a pickup and the corresponding delivery step.

              Please note that either the `jobs` or the `shipments` attribute should be
              specified to build a valid request.

          solution: This attribute is related to the re-optimization feature. It allows for the
              previous optimization result to be provided in case new orders are received and
              the solution needs to be re-planned. The `solution` attribute should contain the
              same routes as the previous optimization result. `solution` attribute is an
              array of objects with each object corresponding to one route.

          unassigned: `unassigned` attribute is related to the re-optimization feature. This attribute
              should contain the tasks that were not assigned during an earlier optimization
              process. Please note that the `unassigned` part in request should be consistent
              with the `unassigned` part in the previous optimization result.

              Users can reduce the number of unassigned tasks in the re-optimized solution, by
              following strategies such as:

              - Extending the time windows for vehicles or tasks to give more flexibility
              - Adding more vehicles to the optimization problem
              - Adjusting the priority of different tasks to balance the workload more evenly
              - Modifying other constraints or parameters to make the problem more solvable

              Ultimately, the goal is to minimize the number of unassigned tasks while still
              meeting all the necessary constraints and objectives.

          zones: An array of objects to specify geometry of all the zones involved. Each object
              corresponds to a single zone. A valid zone can be a
              [geoJSON](https://datatracker.ietf.org/doc/html/rfc7946#page-9) polygon,
              multi-polygon or a geofence created using
              [NextBillion.ai](http://NextBillion.ai)’s
              [Geofence API](https://docs.nextbillion.ai/docs/tracking/api/geofence).

              Please note that

              - Each zone should have a geometry specified either through`geometry` or through
                the `geofence_id` parameter.
              - When zone IDs are not provided for individual tasks (jobs or shipments) then
                the API will automatically allocate zones based on the task’s geolocation and
                the geometries of the zones provided here. Otherwise, if the zone IDs are
                provided while configuring individual tasks, the zone IDs will override the
                geometries provided here.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/optimization/v2",
            body=maybe_transform(
                {
                    "locations": locations,
                    "vehicles": vehicles,
                    "cost_matrix": cost_matrix,
                    "depots": depots,
                    "description": description,
                    "distance_matrix": distance_matrix,
                    "duration_matrix": duration_matrix,
                    "existing_solution_id": existing_solution_id,
                    "jobs": jobs,
                    "options": options,
                    "relations": relations,
                    "shipments": shipments,
                    "solution": solution,
                    "unassigned": unassigned,
                    "zones": zones,
                },
                v2_submit_params.V2SubmitParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, v2_submit_params.V2SubmitParams),
            ),
            cast_to=PostResponse,
        )


class AsyncV2Resource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncV2ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncV2ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncV2ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncV2ResourceWithStreamingResponse(self)

    async def retrieve_result(
        self,
        *,
        id: str,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> V2RetrieveResultResponse:
        """
        Flexible GET

        Args:
          id: The unique ID that was returned on successful submission of the Optimization
              POST request.

          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/optimization/v2/result",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "id": id,
                        "key": key,
                    },
                    v2_retrieve_result_params.V2RetrieveResultParams,
                ),
            ),
            cast_to=V2RetrieveResultResponse,
        )

    async def submit(
        self,
        *,
        key: str,
        locations: v2_submit_params.Locations,
        vehicles: Iterable[VehicleParam],
        cost_matrix: Iterable[Iterable[int]] | NotGiven = NOT_GIVEN,
        depots: Iterable[v2_submit_params.Depot] | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        distance_matrix: Iterable[Iterable[int]] | NotGiven = NOT_GIVEN,
        duration_matrix: Iterable[Iterable[int]] | NotGiven = NOT_GIVEN,
        existing_solution_id: str | NotGiven = NOT_GIVEN,
        jobs: Iterable[JobParam] | NotGiven = NOT_GIVEN,
        options: v2_submit_params.Options | NotGiven = NOT_GIVEN,
        relations: Iterable[v2_submit_params.Relation] | NotGiven = NOT_GIVEN,
        shipments: Iterable[ShipmentParam] | NotGiven = NOT_GIVEN,
        solution: Iterable[v2_submit_params.Solution] | NotGiven = NOT_GIVEN,
        unassigned: v2_submit_params.Unassigned | NotGiven = NOT_GIVEN,
        zones: Iterable[v2_submit_params.Zone] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PostResponse:
        """
        Flexible POST

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          locations: The `locations` object is used to define all the locations that will be used
              during the optimization process. Read more about this attribute in the
              [Location Object](#location-object) section.

          vehicles: The `vehicles` attribute describes the characteristics and constraints of the
              vehicles that will be used for fulfilling the tasks. Read more about this
              attribute in the [Vehicle Object](#vehicle-object) section.

          cost_matrix: An array of arrays to denote the user-defined costs of traveling between each
              pair of geographic coordinates mentioned in the `location` array. The number of
              arrays should be equal to the number of coordinate points mentioned in the
              `location` array and each array should contain the same number of elements as
              well. Please note that `cost_matrix` is effective only when
              `travel_cost=customized`. Read more about this attribute in the
              [Custom Cost Matrix](#custom-cost-matrix) section.

          depots: `depots` object is used to collect the details of a depot. Depots can be used as
              a starting point and/or ending point for the routes and vehicles. They also can
              be used to fulfil pickup and delivery type`jobs` . The loads which are to be
              delivered at task locations will be picked from depots and loads picked-up from
              task locations will be delivered back to the depots. A depot can be configured
              using the following fields:

          description: Define the optimization job using any custom message. This description is
              returned as is in the response.

          distance_matrix: An array of arrays to denote the user-defined distances, in meters, for
              travelling between each pair of geographic coordinates mentioned in the
              `location` array. When this input is provided, actual distances between the
              locations will be ignored in favor of the values provided in this input for any
              distance calculations during the optimization process. The values provided here
              will also be used for cost calculations when `travel_cost` is “distance”.

              The number of arrays in the input should be equal to the number of coordinate
              points mentioned in the `location` array and each array, in turn, should contain
              the same number of elements as well.

              **Note:**

              - `duration_matrix` is mandatory when using`distance_matrix`.
              - When using `distance_matrix` route geometry will not be available in the
                optimized solution.

          duration_matrix: An array of arrays to denote the user-defined durations, in seconds, for
              travelling between each pair of geographic coordinates mentioned in the
              `location` array. When this input is provided, actual durations between the
              locations will be ignored in favor of the values provided in the matrix for any
              ETA calculations during the optimization process. The values provided in the
              matrix will also be used for cost calculations when `travel_cost` is “duration”.

              The number of arrays in the input should be equal to the number of coordinate
              points mentioned in the `location` array and each array, in turn, should contain
              the same number of elements as well.

              Please note that, unlike `distance_matrix`, `duration_matrix` can be used
              independently in following cases:

              - when `travel_cost` is “duration”
              - when `travel_cost` is “customized” and a `cost_matrix` is provided

              Also, the route geometry will not be available in the optimized solution when
              using `duration_matrix`.

          existing_solution_id: The previous optimization request id used to retrieve solution for
              reoptimization

          jobs: `jobs` object is used to collect the details of a particular job or task that
              needs to be completed as part of the optimization process. Each job can have
              either a `pickup` or `delivery` step, but not both. Read more about this
              attribute in the [Job Object](#job-object) section.

              Please note that either the `jobs` or the `shipments` attribute should be
              specified to build a valid request.

          options: It represents the set of options that can be used to configure optimization
              algorithms so that the solver provides a solution that meets the desired
              business objectives.

          relations: `relations` attribute is an array of individual relation objects. `type`
              parameter and `steps` object are mandatory when using this attribute.

              Please note:

              - The soft constraints are **not** effective when using the `relations`
                attribute.
              - In case a given relation can't be satisfied, the optimizer will flag all the
                tasks involved in that "relation" as unassigned.

              Read more about this attribute in the [Relations Object](#relations-object)
              section.

          shipments: The `shipments` object is used to collect the details of shipments that need to
              be completed as part of the optimization process.

              Each shipment should have a pickup and the corresponding delivery step.

              Please note that either the `jobs` or the `shipments` attribute should be
              specified to build a valid request.

          solution: This attribute is related to the re-optimization feature. It allows for the
              previous optimization result to be provided in case new orders are received and
              the solution needs to be re-planned. The `solution` attribute should contain the
              same routes as the previous optimization result. `solution` attribute is an
              array of objects with each object corresponding to one route.

          unassigned: `unassigned` attribute is related to the re-optimization feature. This attribute
              should contain the tasks that were not assigned during an earlier optimization
              process. Please note that the `unassigned` part in request should be consistent
              with the `unassigned` part in the previous optimization result.

              Users can reduce the number of unassigned tasks in the re-optimized solution, by
              following strategies such as:

              - Extending the time windows for vehicles or tasks to give more flexibility
              - Adding more vehicles to the optimization problem
              - Adjusting the priority of different tasks to balance the workload more evenly
              - Modifying other constraints or parameters to make the problem more solvable

              Ultimately, the goal is to minimize the number of unassigned tasks while still
              meeting all the necessary constraints and objectives.

          zones: An array of objects to specify geometry of all the zones involved. Each object
              corresponds to a single zone. A valid zone can be a
              [geoJSON](https://datatracker.ietf.org/doc/html/rfc7946#page-9) polygon,
              multi-polygon or a geofence created using
              [NextBillion.ai](http://NextBillion.ai)’s
              [Geofence API](https://docs.nextbillion.ai/docs/tracking/api/geofence).

              Please note that

              - Each zone should have a geometry specified either through`geometry` or through
                the `geofence_id` parameter.
              - When zone IDs are not provided for individual tasks (jobs or shipments) then
                the API will automatically allocate zones based on the task’s geolocation and
                the geometries of the zones provided here. Otherwise, if the zone IDs are
                provided while configuring individual tasks, the zone IDs will override the
                geometries provided here.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/optimization/v2",
            body=await async_maybe_transform(
                {
                    "locations": locations,
                    "vehicles": vehicles,
                    "cost_matrix": cost_matrix,
                    "depots": depots,
                    "description": description,
                    "distance_matrix": distance_matrix,
                    "duration_matrix": duration_matrix,
                    "existing_solution_id": existing_solution_id,
                    "jobs": jobs,
                    "options": options,
                    "relations": relations,
                    "shipments": shipments,
                    "solution": solution,
                    "unassigned": unassigned,
                    "zones": zones,
                },
                v2_submit_params.V2SubmitParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, v2_submit_params.V2SubmitParams),
            ),
            cast_to=PostResponse,
        )


class V2ResourceWithRawResponse:
    def __init__(self, v2: V2Resource) -> None:
        self._v2 = v2

        self.retrieve_result = to_raw_response_wrapper(
            v2.retrieve_result,
        )
        self.submit = to_raw_response_wrapper(
            v2.submit,
        )


class AsyncV2ResourceWithRawResponse:
    def __init__(self, v2: AsyncV2Resource) -> None:
        self._v2 = v2

        self.retrieve_result = async_to_raw_response_wrapper(
            v2.retrieve_result,
        )
        self.submit = async_to_raw_response_wrapper(
            v2.submit,
        )


class V2ResourceWithStreamingResponse:
    def __init__(self, v2: V2Resource) -> None:
        self._v2 = v2

        self.retrieve_result = to_streamed_response_wrapper(
            v2.retrieve_result,
        )
        self.submit = to_streamed_response_wrapper(
            v2.submit,
        )


class AsyncV2ResourceWithStreamingResponse:
    def __init__(self, v2: AsyncV2Resource) -> None:
        self._v2 = v2

        self.retrieve_result = async_to_streamed_response_wrapper(
            v2.retrieve_result,
        )
        self.submit = async_to_streamed_response_wrapper(
            v2.submit,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from .steps import (
    StepsResource,
    AsyncStepsResource,
    StepsResourceWithRawResponse,
    AsyncStepsResourceWithRawResponse,
    StepsResourceWithStreamingResponse,
    AsyncStepsResourceWithStreamingResponse,
)
from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.fleetify import route_create_params, route_redispatch_params
from ....types.fleetify.route_create_response import RouteCreateResponse
from ....types.fleetify.route_redispatch_response import RouteRedispatchResponse
from ....types.fleetify.routes.route_steps_request_param import RouteStepsRequestParam

__all__ = ["RoutesResource", "AsyncRoutesResource"]


class RoutesResource(SyncAPIResource):
    @cached_property
    def steps(self) -> StepsResource:
        return StepsResource(self._client)

    @cached_property
    def with_raw_response(self) -> RoutesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return RoutesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RoutesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return RoutesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        key: str,
        driver_email: str,
        steps: Iterable[RouteStepsRequestParam],
        distance: int | NotGiven = NOT_GIVEN,
        document_template_id: str | NotGiven = NOT_GIVEN,
        ro_request_id: str | NotGiven = NOT_GIVEN,
        routing: route_create_params.Routing | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RouteCreateResponse:
        """
        Dispatch a new route

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          driver_email: Specify the e-mail address of the driver who should receive the route. The
              e-mail address must be registered in
              [NextBillion.ai Cloud Console](https://console.nextbillion.ai/).

          steps: An array of objects to collect the details about the intermediate steps in the
              route to be dispatched. Each object corresponds to a single step. The array must
              begin with a start-type step and end with an end-type step, to form a valid
              route.

          distance: Specify the total distance, in meters, for an informative display in Driver's
              app. The distance specified here has no effect on the actual route that the
              service generates.

          document_template_id: Specify the ID of the document template that should be used to collect proof of
              completion for all steps in the route. In order to complete each route step, the
              driver will need to submit a form generated by the rules defined in the given
              document template. Use the
              [Documents API](https://docs.nextbillion.ai/docs/dispatches/documents-api) to
              create, read and manage document templates.

              Please note that the document template ID assigned to a route does not apply to
              following step types - `start`, `end`, `break`, `layover`.

          ro_request_id: Specify the Route Optimization request ID. When this ID is provided, all other
              fields will be ignored (including the required fields) and the route
              optimization result will be used to form the routes and corresponding steps.

              Please note that:

              - The driver's email ID must be provided in input `vehicle.metadata` as
                `user_email` such that the route optimization result must contain a valid
                driver email, step's arrival time, etc., to make a successful dispatch.
              - Document Template for collecting proof of delivery or completion can not be
                specified when using this field to dispatch a route.
              - In case of an error at any part among the routes, the API will immediately
                return the error with the index of the specific route or route step.
              - On a successful dispatch, the API returns the last route, if there are many,
                in the response payload.

          routing: The `routing` object allows defining the routing characteristics that should be
              used to generate a route when the Driver uses the in-app navigation. Only `car`
              mode is supported currently.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/fleetify/routes",
            body=maybe_transform(
                {
                    "driver_email": driver_email,
                    "steps": steps,
                    "distance": distance,
                    "document_template_id": document_template_id,
                    "ro_request_id": ro_request_id,
                    "routing": routing,
                },
                route_create_params.RouteCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, route_create_params.RouteCreateParams),
            ),
            cast_to=RouteCreateResponse,
        )

    def redispatch(
        self,
        route_id: str,
        *,
        key: str,
        operations: Iterable[route_redispatch_params.Operation],
        distance: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RouteRedispatchResponse:
        """
        Re-dispatch route

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          operations: A collection of objects with details of the steps to be modified. Each object
              corresponds to a single step.

          distance: Specify the distance of the route.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return self._post(
            f"/fleetify/routes/{route_id}/redispatch",
            body=maybe_transform(
                {
                    "operations": operations,
                    "distance": distance,
                },
                route_redispatch_params.RouteRedispatchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, route_redispatch_params.RouteRedispatchParams),
            ),
            cast_to=RouteRedispatchResponse,
        )


class AsyncRoutesResource(AsyncAPIResource):
    @cached_property
    def steps(self) -> AsyncStepsResource:
        return AsyncStepsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRoutesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRoutesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRoutesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncRoutesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        key: str,
        driver_email: str,
        steps: Iterable[RouteStepsRequestParam],
        distance: int | NotGiven = NOT_GIVEN,
        document_template_id: str | NotGiven = NOT_GIVEN,
        ro_request_id: str | NotGiven = NOT_GIVEN,
        routing: route_create_params.Routing | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RouteCreateResponse:
        """
        Dispatch a new route

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          driver_email: Specify the e-mail address of the driver who should receive the route. The
              e-mail address must be registered in
              [NextBillion.ai Cloud Console](https://console.nextbillion.ai/).

          steps: An array of objects to collect the details about the intermediate steps in the
              route to be dispatched. Each object corresponds to a single step. The array must
              begin with a start-type step and end with an end-type step, to form a valid
              route.

          distance: Specify the total distance, in meters, for an informative display in Driver's
              app. The distance specified here has no effect on the actual route that the
              service generates.

          document_template_id: Specify the ID of the document template that should be used to collect proof of
              completion for all steps in the route. In order to complete each route step, the
              driver will need to submit a form generated by the rules defined in the given
              document template. Use the
              [Documents API](https://docs.nextbillion.ai/docs/dispatches/documents-api) to
              create, read and manage document templates.

              Please note that the document template ID assigned to a route does not apply to
              following step types - `start`, `end`, `break`, `layover`.

          ro_request_id: Specify the Route Optimization request ID. When this ID is provided, all other
              fields will be ignored (including the required fields) and the route
              optimization result will be used to form the routes and corresponding steps.

              Please note that:

              - The driver's email ID must be provided in input `vehicle.metadata` as
                `user_email` such that the route optimization result must contain a valid
                driver email, step's arrival time, etc., to make a successful dispatch.
              - Document Template for collecting proof of delivery or completion can not be
                specified when using this field to dispatch a route.
              - In case of an error at any part among the routes, the API will immediately
                return the error with the index of the specific route or route step.
              - On a successful dispatch, the API returns the last route, if there are many,
                in the response payload.

          routing: The `routing` object allows defining the routing characteristics that should be
              used to generate a route when the Driver uses the in-app navigation. Only `car`
              mode is supported currently.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/fleetify/routes",
            body=await async_maybe_transform(
                {
                    "driver_email": driver_email,
                    "steps": steps,
                    "distance": distance,
                    "document_template_id": document_template_id,
                    "ro_request_id": ro_request_id,
                    "routing": routing,
                },
                route_create_params.RouteCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, route_create_params.RouteCreateParams),
            ),
            cast_to=RouteCreateResponse,
        )

    async def redispatch(
        self,
        route_id: str,
        *,
        key: str,
        operations: Iterable[route_redispatch_params.Operation],
        distance: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RouteRedispatchResponse:
        """
        Re-dispatch route

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          operations: A collection of objects with details of the steps to be modified. Each object
              corresponds to a single step.

          distance: Specify the distance of the route.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return await self._post(
            f"/fleetify/routes/{route_id}/redispatch",
            body=await async_maybe_transform(
                {
                    "operations": operations,
                    "distance": distance,
                },
                route_redispatch_params.RouteRedispatchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, route_redispatch_params.RouteRedispatchParams),
            ),
            cast_to=RouteRedispatchResponse,
        )


class RoutesResourceWithRawResponse:
    def __init__(self, routes: RoutesResource) -> None:
        self._routes = routes

        self.create = to_raw_response_wrapper(
            routes.create,
        )
        self.redispatch = to_raw_response_wrapper(
            routes.redispatch,
        )

    @cached_property
    def steps(self) -> StepsResourceWithRawResponse:
        return StepsResourceWithRawResponse(self._routes.steps)


class AsyncRoutesResourceWithRawResponse:
    def __init__(self, routes: AsyncRoutesResource) -> None:
        self._routes = routes

        self.create = async_to_raw_response_wrapper(
            routes.create,
        )
        self.redispatch = async_to_raw_response_wrapper(
            routes.redispatch,
        )

    @cached_property
    def steps(self) -> AsyncStepsResourceWithRawResponse:
        return AsyncStepsResourceWithRawResponse(self._routes.steps)


class RoutesResourceWithStreamingResponse:
    def __init__(self, routes: RoutesResource) -> None:
        self._routes = routes

        self.create = to_streamed_response_wrapper(
            routes.create,
        )
        self.redispatch = to_streamed_response_wrapper(
            routes.redispatch,
        )

    @cached_property
    def steps(self) -> StepsResourceWithStreamingResponse:
        return StepsResourceWithStreamingResponse(self._routes.steps)


class AsyncRoutesResourceWithStreamingResponse:
    def __init__(self, routes: AsyncRoutesResource) -> None:
        self._routes = routes

        self.create = async_to_streamed_response_wrapper(
            routes.create,
        )
        self.redispatch = async_to_streamed_response_wrapper(
            routes.redispatch,
        )

    @cached_property
    def steps(self) -> AsyncStepsResourceWithStreamingResponse:
        return AsyncStepsResourceWithStreamingResponse(self._routes.steps)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, TypedDict

from .job_param import JobParam
from .vehicle_param import VehicleParam
from .shipment_param import ShipmentParam

__all__ = [
    "V2SubmitParams",
    "Locations",
    "Depot",
    "Options",
    "OptionsConstraint",
    "OptionsGrouping",
    "OptionsGroupingOrderGrouping",
    "OptionsGroupingRouteGrouping",
    "OptionsObjective",
    "OptionsObjectiveCustom",
    "OptionsRouting",
    "Relation",
    "RelationStep",
    "Solution",
    "SolutionStep",
    "Unassigned",
    "Zone",
    "ZoneGeometry",
]


class V2SubmitParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    locations: Required[Locations]
    """
    The `locations` object is used to define all the locations that will be used
    during the optimization process. Read more about this attribute in the
    [Location Object](#location-object) section.
    """

    vehicles: Required[Iterable[VehicleParam]]
    """
    The `vehicles` attribute describes the characteristics and constraints of the
    vehicles that will be used for fulfilling the tasks. Read more about this
    attribute in the [Vehicle Object](#vehicle-object) section.
    """

    cost_matrix: Iterable[Iterable[int]]
    """
    An array of arrays to denote the user-defined costs of traveling between each
    pair of geographic coordinates mentioned in the `location` array. The number of
    arrays should be equal to the number of coordinate points mentioned in the
    `location` array and each array should contain the same number of elements as
    well. Please note that `cost_matrix` is effective only when
    `travel_cost=customized`. Read more about this attribute in the
    [Custom Cost Matrix](#custom-cost-matrix) section.
    """

    depots: Iterable[Depot]
    """`depots` object is used to collect the details of a depot.

    Depots can be used as a starting point and/or ending point for the routes and
    vehicles. They also can be used to fulfil pickup and delivery type`jobs` . The
    loads which are to be delivered at task locations will be picked from depots and
    loads picked-up from task locations will be delivered back to the depots. A
    depot can be configured using the following fields:
    """

    description: str
    """Define the optimization job using any custom message.

    This description is returned as is in the response.
    """

    distance_matrix: Iterable[Iterable[int]]
    """
    An array of arrays to denote the user-defined distances, in meters, for
    travelling between each pair of geographic coordinates mentioned in the
    `location` array. When this input is provided, actual distances between the
    locations will be ignored in favor of the values provided in this input for any
    distance calculations during the optimization process. The values provided here
    will also be used for cost calculations when `travel_cost` is “distance”.

    The number of arrays in the input should be equal to the number of coordinate
    points mentioned in the `location` array and each array, in turn, should contain
    the same number of elements as well.

    **Note:**

    - `duration_matrix` is mandatory when using`distance_matrix`.
    - When using `distance_matrix` route geometry will not be available in the
      optimized solution.
    """

    duration_matrix: Iterable[Iterable[int]]
    """
    An array of arrays to denote the user-defined durations, in seconds, for
    travelling between each pair of geographic coordinates mentioned in the
    `location` array. When this input is provided, actual durations between the
    locations will be ignored in favor of the values provided in the matrix for any
    ETA calculations during the optimization process. The values provided in the
    matrix will also be used for cost calculations when `travel_cost` is “duration”.

    The number of arrays in the input should be equal to the number of coordinate
    points mentioned in the `location` array and each array, in turn, should contain
    the same number of elements as well.

    Please note that, unlike `distance_matrix`, `duration_matrix` can be used
    independently in following cases:

    - when `travel_cost` is “duration”
    - when `travel_cost` is “customized” and a `cost_matrix` is provided

    Also, the route geometry will not be available in the optimized solution when
    using `duration_matrix`.
    """

    existing_solution_id: str
    """
    The previous optimization request id used to retrieve solution for
    reoptimization
    """

    jobs: Iterable[JobParam]
    """
    `jobs` object is used to collect the details of a particular job or task that
    needs to be completed as part of the optimization process. Each job can have
    either a `pickup` or `delivery` step, but not both. Read more about this
    attribute in the [Job Object](#job-object) section.

    Please note that either the `jobs` or the `shipments` attribute should be
    specified to build a valid request.
    """

    options: Options
    """
    It represents the set of options that can be used to configure optimization
    algorithms so that the solver provides a solution that meets the desired
    business objectives.
    """

    relations: Iterable[Relation]
    """`relations` attribute is an array of individual relation objects.

    `type` parameter and `steps` object are mandatory when using this attribute.

    Please note:

    - The soft constraints are **not** effective when using the `relations`
      attribute.
    - In case a given relation can't be satisfied, the optimizer will flag all the
      tasks involved in that "relation" as unassigned.

    Read more about this attribute in the [Relations Object](#relations-object)
    section.
    """

    shipments: Iterable[ShipmentParam]
    """
    The `shipments` object is used to collect the details of shipments that need to
    be completed as part of the optimization process.

    Each shipment should have a pickup and the corresponding delivery step.

    Please note that either the `jobs` or the `shipments` attribute should be
    specified to build a valid request.
    """

    solution: Iterable[Solution]
    """This attribute is related to the re-optimization feature.

    It allows for the previous optimization result to be provided in case new orders
    are received and the solution needs to be re-planned. The `solution` attribute
    should contain the same routes as the previous optimization result. `solution`
    attribute is an array of objects with each object corresponding to one route.
    """

    unassigned: Unassigned
    """`unassigned` attribute is related to the re-optimization feature.

    This attribute should contain the tasks that were not assigned during an earlier
    optimization process. Please note that the `unassigned` part in request should
    be consistent with the `unassigned` part in the previous optimization result.

    Users can reduce the number of unassigned tasks in the re-optimized solution, by
    following strategies such as:

    - Extending the time windows for vehicles or tasks to give more flexibility
    - Adding more vehicles to the optimization problem
    - Adjusting the priority of different tasks to balance the workload more evenly
    - Modifying other constraints or parameters to make the problem more solvable

    Ultimately, the goal is to minimize the number of unassigned tasks while still
    meeting all the necessary constraints and objectives.
    """

    zones: Iterable[Zone]
    """An array of objects to specify geometry of all the zones involved.

    Each object corresponds to a single zone. A valid zone can be a
    [geoJSON](https://datatracker.ietf.org/doc/html/rfc7946#page-9) polygon,
    multi-polygon or a geofence created using
    [NextBillion.ai](http://NextBillion.ai)’s
    [Geofence API](https://docs.nextbillion.ai/docs/tracking/api/geofence).

    Please note that

    - Each zone should have a geometry specified either through`geometry` or through
      the `geofence_id` parameter.
    - When zone IDs are not provided for individual tasks (jobs or shipments) then
      the API will automatically allocate zones based on the task’s geolocation and
      the geometries of the zones provided here. Otherwise, if the zone IDs are
      provided while configuring individual tasks, the zone IDs will override the
      geometries provided here.
    """


class Locations(TypedDict, total=False):
    location: Required[List[str]]
    """Indicate all the location coordinates that will be used during optimization.

    The coordinates should be specified in the format “latitude, longitude”. It is
    recommended to avoid adding duplicate location coordinates to this array. In
    case there are multiple tasks at the same location, users can repeat the index
    of the location while configuring all such tasks.

    Please use this array to determine the index of a location when setting the
    `location_index` parameter in `jobs`, `shipments`, `vehicles` or other parts of
    the request. The length of this array determines the valid values for
    `location_index` parameter.
    """

    id: int
    """A unique ID for the set of locations. It should be a positive integer."""

    approaches: List[Literal["`unrestricted`", "`curb`", '""(empty string)']]
    """Describe if the location is curbside.

    An array of strings indicates the side of the road from which to approach the
    location in the calculated route. If provided, the number of approaches must be
    equal to the number of locations. However, you can skip a coordinate and show
    its position in the list using “” (empty string). Please note these values are
    case-sensitive.
    """


class Depot(TypedDict, total=False):
    id: Required[str]
    """Provide an unique ID for the depot. The IDs are case sensitive."""

    location_index: Required[int]
    """
    Specify the index of coordinates (in the `location` array) denoting the depot’s
    location. The valid range of values is \\[[0, length of `location` array). If the
    location index exceeds the count of input locations in the `location` array, the
    API will report an error.

    Please note the `location_index` is mandatory when using the `depots` object.
    """

    description: str
    """Add a custom description for the depot."""

    service: int
    """
    Specify the time duration, in seconds, needed to load or unload the vehicle each
    time it starts or arrives at a depot, respectively. Default value is 0.
    """

    time_windows: Iterable[Iterable[int]]
    """
    Specify the time-windows during which the depot is operational and allows
    vehicles to be loaded / unloaded. The time periods should be expressed as a UNIX
    timestamp in seconds.

    Please note that:

    - Multiple time-windows can be provided but those time windows should not
      overlap with each other.
    - Time windows should always be specified in the format of \\[[start_timestamp,
      end_timestamp\\]].
    - Depot's time-windows are ineffective used when `max_activity_waiting_time` is
      specified in the input.
    - Using `relations` along with depot time-window is not allowed and the service
      will return an error.
    """


class OptionsConstraint(TypedDict, total=False):
    max_activity_waiting_time: int
    """
    This is a hard constraint which specifies the maximum waiting time, in seconds,
    for each `step`. It ensures that the vehicles do not have unreasonable wait
    times between jobs or shipments. This feature is useful for use cases where
    avoiding long wait times between jobs or shipments is a primary concern.

    Please note that the waiting time constraint applies to all tasks in the
    optimization request, ensuring that no single task exceeds the specified maximum
    waiting time. When being used together with `relations` attribute, this
    parameter is effective only for `in_same_route` relation type.
    """

    max_vehicle_overtime: int
    """This is a soft constraint for vehicle overtime.

    Overtime is defined as the time that a vehicle spends to complete a set of jobs
    after its time window has ended. `max_vehicle_overtime` attribute specifies the
    maximum amount of overtime a vehicle can have, in seconds. If a vehicle’s
    overtime exceeds this value, it will be considered a violation of this
    constraint.

    Please note that this constraint applies to all vehicles in the optimization
    request.
    """

    max_visit_lateness: int
    """
    This is a soft constraint for permissible delay, in seconds, to complete a job
    or shipment after its time window is over. If a job or shipment’s lateness
    exceeds this value, it will be considered a violation of this constraint.

    Please note that this constraint applies to all tasks in the optimization
    request. In case lateness duration needs to be applied for individual tasks,
    please use the `max_visit_lateness` parameter under `jobs` and `shipments`
    """


class OptionsGroupingOrderGrouping(TypedDict, total=False):
    grouping_diameter: float
    """
    Specify the straight line distance, in meters, which will be used to identify
    the tasks that should be grouped together. The default value is `null`.
    """


class OptionsGroupingRouteGrouping(TypedDict, total=False):
    penalty_factor: float
    """
    Specify a non-negative value which indicates the penalty of crossing zones on
    the same route. Default penalty value is 0.

    A higher value, for example 30.0, will place a higher penalty on zone violations
    and hence push the optimizer to prefer a solution without any zone violations,
    where all tasks in a single region are fulfilled before any tasks in other
    regions or outside the current region. Whereas a lower value, say 5.0, will
    place a lower penalty allowing the optimizer to return solutions which may have
    few violations, say a couple of routing zone violations in our example. A still
    lower penalty factor, like 1.0, may have several zone violations.
    """

    zone_diameter: float
    """
    Specify the diameter of the zone, routes within which will be prioritised before
    routes falling in other zones. Please note that `zone_diameter` is the straight
    line distance, in meters.
    """

    zone_source: Literal["`system_generated`", "`custom_definition`"]
    """Specify the source for creating boundaries of the routing zones.

    The default value is “system_generated”.

    - system_generated - Routing zone boundaries are created automatically by the
      optimizer based on the `zone_diameter` provided.
    - custom_definition - Custom routing zone boundaries should be provided by the
      user in input using the `zones` attribute. An error would be returned if the
      `zones` attribute is null or missing in the input request.
    """


class OptionsGrouping(TypedDict, total=False):
    order_grouping: OptionsGroupingOrderGrouping
    """Specify the criteria for grouping nearby tasks.

    The grouped tasks will be treated as one stop by the optimizer and no cost would
    be incurred when driver travels to different tasks within a group. Users can use
    this feature to model use cases like multiple deliveries in a building complex
    or a condo.

    Please note that when the multiple tasks are grouped together, only one setup
    time is considered for all such tasks. The durations of this setup time is equal
    to maximum setup time among all grouped tasks, if provided. On the other hand,
    the service time is applied to each task individually, as per the input provided
    when configuring those tasks.
    """

    proximity_factor: float
    """
    When specified, routes are built taking into account the distance to the nearest
    tasks. A higher proximity factor helps build routes with closer distances
    between neighboring tasks, whereas a lower proximity factor helps build routes
    with farther distances between neighboring tasks. As a result, the total number
    of routes in the solution can vary based on the configured proximity factor -
    more routes for higher factor and less routes with lower factor.

    In practice, such routes are more resistant to changes in task time windows:
    when the time window is postponed, the driver can drive to the next task and
    then return to the previous one.

    Please note that:

    - Valid values are \\[[0,10\\]]
    - Default value is 0.0.
    - It is recommended to use values lower values, in the range of \\[[0, 1\\]]. Higher
      values may adversely impact the solution metrics due to higher number of
      resulting routes: costs, mileage etc.
    """

    route_grouping: OptionsGroupingRouteGrouping
    """
    Specify the criteria for prioritising routes in a zone over routes that are part
    of another zone. As a result, all the tasks falling in a zone will be fulfilled
    before any tasks that are part of a different zone.
    """


class OptionsObjectiveCustom(TypedDict, total=False):
    type: Required[Literal["`min`", "`min-max`"]]
    """The `type` parameter accepts two inputs:

    - `min`: This type of customized objective will minimize the metric provided in
      the `value` parameter.
    - `min-max`: This type of customized objective will approximate an even
      distribution of the metric provided in the `value` parameter, among all the
      routes in solution.

    Please note that `type` is mandatory only when using `custom` attribute.
    """

    value: Required[Literal["`vehicles`", "`completion_time`", "`travel_cost`", "`tasks`"]]
    """
    The `value` parameter accepts four inputs, two of them are valid for `min` type
    and other two are valid for `min-max` type custom objective. Let’s look at the
    values for `min` type objective:

    - `vehicles`: Solver will minimize the number of vehicles used in the solution.
    - `completion_time`: Solver will minimize the total time taken to complete all
      tasks.

    The next set of values are acceptable when `type` is set to `min-max`.

    - `tasks`: Solver will evenly distribute the tasks on each route.
    - `travel_cost`: Solver will assign tasks such that the traveling cost of each
      route is within a close range of other routes. The travel cost metric
      considered here is the one set using `objective.travel_cost` .

    Please note that `value` is mandatory only when using `custom` attribute. The
    above values provide flexibility to tune the optimization algorithm to fulfill
    practical objectives beyond the relatively simpler time or distance minimization
    approaches.
    """


class OptionsObjective(TypedDict, total=False):
    allow_early_arrival: bool
    """Choose where the optimizer should schedule the driver’s wait time.

    When set to `true` the driver waits at the location of the task until its time
    window allows him to start the task. When set to `false` the driver waits at the
    location of the previous task and starts driving only at such a time that makes
    him arrive at the next task location in time to start the task as soon as he
    reaches.
    """

    custom: OptionsObjectiveCustom
    """
    The `custom` parameter is used to define special objectives apart from the
    simpler travel cost minimization objectives.
    """

    minimise_num_depots: bool
    """Specify whether to minimize the number of depots used in optimization routes."""

    solver_mode: Literal["`flexible`", "`fast`", "`internal`"]
    """
    If the input doesn’t include features of soft constraints, customized
    objectives, re-optimization, relations, max travel cost or automatic fixed cost,
    then user can select “optimal” to achieve a higher-speed and higher-quality
    optimization.

    If “optimal” mode is unable to process some features in the input, then it will
    still goes to “flexible” mode.
    """

    solving_time_limit: int
    """
    Specify the number of seconds within which the optimizer should ideally solve
    the optimization request.

    Please note that:

    - In case the specified time limit is not enough to generate a solution for a
      given problem set, the optimizer will continue processing until it arrives at
      a solution.
    - It is recommended to specify a duration of at least 5-7 minutes in case the
      input problem contains a large set of tasks or vehicles.
    """

    travel_cost: Literal["`duration`", "`distance`", "`air_distance`", "`customized`"]
    """
    The `travel_cost` parameter specifies the type of cost used by the solver to
    determine the routes.

    If the `travel_cost` parameter is set to `distance`, the solver will minimize
    the total distance traveled by vehicles while determining a solution. This
    objective would be useful in cases where the primary objective is to reduce fuel
    consumption or travel expenses.

    If the `travel_cost` parameter is set to `duration`, the solver will minimize
    the total time taken by the vehicles to complete all tasks while determining a
    solution. This objective would be useful in cases where the primary objective is
    to minimize completion time or maximize the number of orders fulfilled within a
    given time window.

    If the `travel_cost` parameter is set to `air_distance`, the solver will try to
    calculate the distance,in meters, between two points using the great-circle
    distance formula (i.e., the shortest distance between two points on a sphere)
    instead of the actual road distance. This would be useful in cases where the
    delivery locations are far apart and the road distance between them is
    significantly longer than the actual straight-line distance. For example, in
    Drone Delivery services.

    If the `travel_cost` is set to `customized` the solver would use the custom cost
    values provided by the user (in `cost_matrix` attribute) and prefer a solution
    with lower overall cost. This enables the user to have greater control over the
    routes preferred by the solver and hence the sequence in which the jobs are
    completed.
    """


class OptionsRouting(TypedDict, total=False):
    allow: List[Literal["taxi", "hov"]]

    avoid: List[
        Literal[
            "`toll`",
            "`highway`",
            "`bbox`",
            "`left_turn`",
            "`right_turn`",
            "`sharp_turn`",
            "`uturn`",
            "`service_road`",
            "`ferry`",
            "`none` ",
        ]
    ]
    """Specify the type of objects/maneuvers that the route should avoid.

    Please note that:

    - The values are case-sensitive.
    - When using `avoid:bbox` feature, users need to specify the boundaries of the
      bounding box to be avoided. Multiple bounding boxes can be provided
      simultaneously. Please note that bounding box is a hard filter and if it
      blocks all possible routes between given locations, a 4xx error is returned.
      Mention the bounding box boundaries in the following format: bbox:
      min_latitude,min_longitude,max_latitude,max_longitude.
    - When using `avoid=sharp_turn`, the range of allowed turn angles is \\[[120,240\\]]
      in the clockwise direction from the current road. Any roads with turn angles
      outside the range will be avoided.
    - If `none` is provided along with other values, an error is returned as a valid
      route is not feasible.
    """

    context: Literal["`avgspeed`"]
    """
    Use this parameter to apply a single speed value for all ETA and drive time
    calculations. In case, the `travel_cost` is set to duration then setting this
    parameter also impacts the cost of the solution.
    """

    cross_border: bool
    """
    Specify if crossing an international border is allowed for operations near
    border areas. When set to false, the API will prohibit any routes crossing
    international borders. When set to true, the service will return routes which
    cross the borders between countries, if required for the given set `locations`

    This feature is available in North America region only. Please get in touch with
    [support@nextbillion.ai](mailto:support@nextbillion.ai) to enquire/enable other
    areas.
    """

    disable_cache: bool
    """
    Specify if the optimizer should cache the matrix result set (containing ETAs and
    distances) for the given set of locations in the request. Once the results are
    cached, the optimizer can use it during the next 60 mins if exactly the same set
    of locations are provided again. Please note that if a cached result is
    retrieved, the timer is reset and that result will be available for another 60
    mins.

    If the users want to regenerate the result set, they can set this parameter to
    `true` and optimizer will not use the cached results.

    This feature is helpful in expediting the optimization process and generate
    results quickly. It also helps users to quickly simulate route plans for
    different combinations of constraints for a given set of locations.
    """

    hazmat_type: List[Literal["`general`", "`circumstantial`", "`explosive`", "`harmful_to_water`"]]
    """
    Specify the type of hazardous material being carried and the service will avoid
    roads which are not suitable for the type of goods specified. Provide multiple
    values separated by a comma `,` .

    Please note that this parameter is effective only when `mode=truck`.
    """

    mode: Literal["`car`", "`truck`"]
    """Define the traveling mode to be used for determining the optimized routes."""

    profiles: object
    """Defines all the vehicle profiles.

    `profiles` is implemented as a dictionary of objects where each profile name is
    the unique key and the associated value is an object describing the routing
    properties of that profile. All routing properties available in
    `options.routing` can be added as values for a given profile.

    Please note:

    - The routing properties configured using `options.routing` (and not part of any
      \\``profiles\\``) are considered as default route settings i.e. they are applied
      to vehicles which are not associated with any profile.
    - The default route settings are independent from those defined for any
      `profiles` . Consequently, for vehicles which are tagged to a given profile,
      only the routing properties configured for the given profile will apply.
    - If the "mode" is not specified for any profile, by default it is considered to
      be `car` .
    - "default" is a reserved keyword and can not be used as the name for any custom
      profile.
    - `profiles` can't be nested in other profiles.
    - The number of profiles, including default route settings, are limited to

      - 15, if 0 < number of location <= 100
      - 6, if 100 < number of location <= 600，
      - 2, if 600 < number of location <= 1200,
      - 1, if number of location > 1200

    Routing profiles attribute is useful for configuring fleets containing multiple
    vehicles types. Check
    [Routing Profiles](https://docs.nextbillion.ai/docs/optimization/api/route-optimization-flexible/tutorials/routing-profiles)
    tutorial to learn more.
    """

    traffic_timestamp: int
    """Specify the general time when the job needs to be carried out.

    The time should be expressed as an UNIX timestamp in seconds format. The solver
    will take into account the general traffic conditions at the given time to
    determine the routes and their ETAs.
    """

    truck_axle_load: float
    """
    Specify the total load per axle (including the weight of trailers and shipped
    goods) of the truck, in tonnes. When used, the optimizer will use only those
    routes which are legally allowed to carry the load specified per axle.

    Please note this parameter is effective only when `mode=truck`.
    """

    truck_size: str
    """
    Specify the truck dimensions, in centimeters, in the format of
    “height,width,length”. Please note that this parameter is effective only when
    `mode=truck`.
    """

    truck_weight: int
    """Specify the truck weight including the trailers and shipped goods, in kilograms.

    Please note that this parameter is effective only when `mode=truck`.
    """


class Options(TypedDict, total=False):
    constraint: OptionsConstraint
    """
    This attribute defines both the soft and hard constraints for an optimization
    job.

    Soft constraints are constraints that do not necessarily have to be satisfied,
    but the optimization algorithm will try to satisfy them as much as possible.
    Whereas the hard constraints are the constraints that will not be violated by
    the solver. Users can use multiple constraints together.

    Please note that soft constraints are ineffective when using `relations`
    attribute in a request. The hard constraint, `max_activity_waiting_time`, is
    effective only when relation type is `in_same_route` and ineffective for all
    other types.
    """

    grouping: OptionsGrouping
    """Set grouping rules for the tasks and routes.

    - Use `order_grouping` to group nearby tasks
    - Use `route_grouping` to control route sequencing.
    """

    objective: OptionsObjective
    """This attribute is used to configure the objective of the optimization job."""

    routing: OptionsRouting
    """
    This attribute is used to define the routing configurations for the optimization
    job.
    """


class RelationStep(TypedDict, total=False):
    type: Required[Literal["`start`", "`end`", "`job`", "`pickup`", "`delivery`"]]
    """Specifies the type of the step.

    The `start` and `end` step types have to be the first and last steps,
    respectively, in a relation.

    Please note that the `type` is mandatory when using the `relations` object.
    """

    id: str
    """
    This represents the ID of the task and should be consistent with the input IDs
    provided in the `jobs` or `shipments` objects for a given step. The `id` is
    required for all steps other than `start` and `end`.
    """


class Relation(TypedDict, total=False):
    steps: Required[Iterable[RelationStep]]
    """
    The `steps` property specifies the tasks or steps that are part of the relation
    and must be carried out in a manner defined in the `type` parameter. Please note
    you can add any number of steps here, except when relation type is `precedence`
    where only 2 tasks can be added.
    """

    type: Required[Literal["`in_same_route`", "`in_sequence`", "`in_direct_sequence`", "`precedence`"]]
    """Specifies the type of relation constraint. The following types are supported:

    - `in_same_route`: Ensures that all `steps` are covered in the same route in
      solution.
    - `in_sequence`: Ensures that all steps are in the same route and their sequence
      matches the order specified in the `steps` field. Insertion of new steps
      between the `steps` specified, is allowed.
    - `in_direct_sequence`: Similar to `in_sequence`, but insertion of new `steps`
      is not allowed in the final route.
    - `precedence`: Restricts the travel time between the first step and second
      step. If the precedence requirement cannot be satisfied, then the task
      specified at the second step will not be assigned. Only 2 steps can be
      specified in a single `precedence` type relations. Please use multiple
      `precedence` relations to apply restrictions on more than 2 tasks.

    If the `vehicle` field is specified in the relations input, all steps will be
    served by that particular vehicle. Otherwise, the route can be allocated to any
    feasible vehicle.

    Please note that the `type` field is mandatory when using the `relations`
    object.
    """

    id: int
    """
    **Deprecated! Please use the** `vehicle` **parameter to specify the vehicle
    ID.**

    Specifies the ID of the vehicle that would fulfil the steps. ID should be
    consistent with input IDs provided in the `vehicles` object.
    """

    max_duration: int
    """This attribute is effective only when `precedence` type relation is used.

    `max_duration` restricts the travel time of the vehicle to go from location of
    first task to the location of second task specified in `steps` object. The unit
    for this parameter is seconds. It accepts values greater than 0 only.

    Please note that `max_duration` is a hard constraint. Hence, if aggressive
    durations are provided such that the second task cannot be reached within the
    specified `max_duration`, it might be done before the first task (usually in
    case of `jobs`) or remain un-assigned (usually in case of `shipments`).
    """

    min_duration: int
    """This attribute is effective only when `precedence` type relation is used.

    Use `min_duration` to enforce a minimum time-gap between the two tasks specified
    in `steps` object. When specified, the second task will get completed after a
    gap of `min_duration` with respect to the first task. The unit for this
    parameter is seconds.

    Please note that `min_duration` is implemented as a soft constraint and it can
    be violated in presence of other relation types. The optimizer will tend to
    provide solutions where `min_duration` is not violated, but it is not
    guaranteed.
    """

    vehicle: str
    """Specifies the ID of the vehicle that would fulfill the steps.

    Providing the same vehicle ID to multiple ‘relations’ is prohibited. The vehicle
    ID provided here should be consistent with ID provided in the `vehicles`
    attribute.
    """


class SolutionStep(TypedDict, total=False):
    id: Required[str]
    """The ID of the step.

    This field is mandatory for all steps except for `start` and `end` type.

    Please note that the ID provided here must also be present in either the `jobs`
    or the `shipments` objects.

    **Note:** We have modified the data type of this field. The latest change is
    backward compatible and both integer and string type IDs are valid for this
    field, as long as they match the IDs of the jobs or shipments already
    configured.
    """

    arrival: Required[int]
    """Specify the time at which the vehicle arrives at the `step` location.

    If `time_windows` is provided, then `arrival` will be an UNIX timestamp
    expressed in seconds. Otherwise, it will be the total duration, in seconds,
    elapsed since the start of the route.

    Please note that arrival is mandatory when using the `solution` object.
    """

    type: Required[Literal["`start`", "`end`", "`job`", "`pickup`", "`delivery`", "`break`"]]
    """Specify the type of the step."""

    description: str
    """Specify the description of this step."""

    distance: int
    """
    Specify the distance covered, in meters, from the start of the route up until
    the current step.

    Please note that the value of this parameter accumulates with each step. In case
    , the `travel_cost: air_distance`, then the distance here should be the straight
    line distance.
    """

    duration: int
    """
    Specify the drive time, in seconds, from the start of the route up until the
    start of the `step`. Please note that the value of this parameter accumulates
    with each step.
    """

    load: Iterable[int]
    """Specify the load on the vehicle after completing this step.

    In case of multiple dimensions, please specify the load for each type.
    """

    location: Iterable[float]
    """
    Specify the location coordinates of the step in the \\[[latitude, longitude\\]]
    format. Alternatively, `location_index` property can also be used to specify the
    location of the step.

    Please note that either `location` or `location_index` is mandatory.
    """

    location_index: int
    """
    Specify the index (in the `location` array) of the location coordinates where
    the step is performed. The valid range of values is \\[[0, length of `location`
    array). Alternatively, `location` property can also be used to specify the
    location.

    Please note that either `location` or `location_index` is mandatory.
    """

    service: int
    """Specify the service time, in seconds, at this step."""

    setup: int
    """Specify the set-up duration, in seconds, needed at the step."""

    waiting_time: int
    """Specify the wait time of the vehicle at this step, in seconds."""


class Solution(TypedDict, total=False):
    cost: Required[int]
    """Specify the cost of the route."""

    steps: Required[Iterable[SolutionStep]]
    """Describe the steps in this route."""

    vehicle: Required[str]
    """Specify the ID of the vehicle that was assigned to the route.

    This field is mandatory when using the `solution` attribute and providing an
    empty string would result in error. The IDs are case-sensitive.

    **Note:** Since the vehicles can be configured using either a string or an
    integer ID, please ensure that the same value type is provided for this field as
    was used in the original request.
    """

    delivery: Iterable[int]
    """
    Specify the total quantities, for each dimension (or unit), of deliveries
    performed in the route.
    """

    description: str
    """Specify the description of the assigned vehicle."""

    distance: int
    """Specify the total distance of the route, in meters."""

    duration: int
    """Specify the total drive duration of the route, in seconds."""

    geometry: str
    """Specify the geometry of this route encoded in polyline format."""

    pickup: Iterable[int]
    """
    Specify the total quantities, for each dimension (or unit), of pickups performed
    in the route.
    """

    priority: int
    """Specify the sum of priorities of all tasks on the route."""

    service: int
    """Specify the total service time for the route, in seconds."""

    setup: int
    """
    Specify the total set-up duration, in seconds, needed for the tasks on the
    route.
    """

    waiting_time: int
    """Specify the total waiting time of the vehicle on the route, in seconds."""


class Unassigned(TypedDict, total=False):
    jobs: List[str]
    """Specify the unassigned job IDs from the previous optimization result.

    Please note the IDs should also be present in the `jobs` part of the input.

    **Note:** We have modified the data type of this field. However, the latest
    change is backward compatible and both integer and string type job IDs are valid
    for this field, as long as they match the IDs of the jobs already configured.
    Providing mixed value types in the array, will lead to an error.
    """

    shipments: Iterable[List[str]]
    """
    Specify the unassigned shipment pickup & delivery IDs from the previous
    optimization result. Both the pickup & delivery steps of a shipment should be
    part of the same array.

    **Note:** We have modified the data type of this field. However, the latest
    change is backward compatible and both integer and string type shipment IDs are
    valid for this field, as long as they match the IDs of the shipments already
    configured. Providing mixed value types in the array, will lead to an error.
    """


class ZoneGeometry(TypedDict, total=False):
    coordinates: Iterable[Iterable[float]]
    """
    An array of coordinates in the \\[[longitude, latitude\\]] format, representing the
    zone boundary.
    """

    description: str
    """Provide a description to identify the zone"""

    type: Literal["`Polygon`", "`MultiPolygon`"]
    """Type of the geoJSON geometry. Should always be `Polygon` or `MultiPolygon`."""


class Zone(TypedDict, total=False):
    id: Required[int]
    """Provide an ID for the zone. This field is mandatory when adding zones."""

    geofence_id: str
    """
    Provide the ID of a pre-created geofence using the
    [Geofence API](https://docs.nextbillion.ai/docs/tracking/api/geofence).

    Please note that one of `geometry` or `geofence_id` should be provided.
    """

    geometry: ZoneGeometry
    """
    It is a [geoJSON object](https://datatracker.ietf.org/doc/html/rfc7946#page-9)
    with details of the geographic boundaries of the zone. Only “Polygon” and
    “MultiPolygon” geoJSON types are supported.

    Please note that one of `geometry` or `geofence_id` should be provided.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .multigeocode.place_item import PlaceItem

__all__ = ["MultigeocodeSearchResponse", "Entity", "EntityDataSource"]


class EntityDataSource(BaseModel):
    ref_id: Optional[str] = FieldInfo(alias="refId", default=None)
    """
    This parameter represents the unique reference ID associated with the data
    source.
    """

    source: Optional[str] = None
    """This parameter represents the source of the data."""

    status: Optional[Literal["enable", "disable"]] = None
    """This parameter indicates if a place is searchable."""


class Entity(BaseModel):
    data_source: Optional[EntityDataSource] = FieldInfo(alias="dataSource", default=None)
    """It contains information about the dataset that returns the specific result"""

    doc_id: Optional[str] = FieldInfo(alias="docId", default=None)
    """The unique NextBillion ID for the result item.

    This ID can be used as input in “Get Place”, “Update Place”, “Delete Place”
    methods.
    """

    place: Optional[List[PlaceItem]] = None
    """
    This parameter represents the place details, including geographical information,
    address and other related information.
    """

    score: Optional[int] = None
    """Integer value representing how good the result is.

    Higher score indicates a better match between the search query and the result.
    This can be used to accept or reject the results depending on how “relevant” a
    result is, for a given use case
    """


class MultigeocodeSearchResponse(BaseModel):
    entities: Optional[List[Entity]] = None
    """An array of objects containing the search result response.

    Each object represents one place returned in the search response. An empty array
    would be returned if no matching place is found.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["Position"]


class Position(BaseModel):
    lat: Optional[str] = None
    """The latitude of the searched place."""

    lng: Optional[str] = None
    """The longitude of the searched place."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .track_location import TrackLocation as TrackLocation
from .event_list_params import EventListParams as EventListParams
from .event_list_response import EventListResponse as EventListResponse
from .location_list_params import LocationListParams as LocationListParams
from .location_list_response import LocationListResponse as LocationListResponse
from .location_get_last_params import LocationGetLastParams as LocationGetLastParams
from .location_get_last_response import LocationGetLastResponse as LocationGetLastResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .document_template_content_response import DocumentTemplateContentResponse

__all__ = ["DocumentTemplateUpdateResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Returns the unique ID of the document template."""

    content: Optional[List[DocumentTemplateContentResponse]] = None
    """
    An array of object returning the details of updated data structures and
    validation rules for document fields. Each object represents one document field.
    """

    name: Optional[str] = None
    """Returns the updated name of the document template."""


class DocumentTemplateUpdateResponse(BaseModel):
    data: Optional[Data] = None
    """An object returning the details of the updated document template."""

    msg: Optional[str] = None
    """Returns the error message in case of a failed request.

    If the request is successful, this field is not present in the response.
    """

    status: Optional[int] = None
    """Returns the HTTP response code."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.fleetify import (
    document_template_list_params,
    document_template_create_params,
    document_template_delete_params,
    document_template_update_params,
    document_template_retrieve_params,
)
from ...types.fleetify.document_template_list_response import DocumentTemplateListResponse
from ...types.fleetify.document_template_create_response import DocumentTemplateCreateResponse
from ...types.fleetify.document_template_delete_response import DocumentTemplateDeleteResponse
from ...types.fleetify.document_template_update_response import DocumentTemplateUpdateResponse
from ...types.fleetify.document_template_retrieve_response import DocumentTemplateRetrieveResponse
from ...types.fleetify.document_template_content_request_param import DocumentTemplateContentRequestParam

__all__ = ["DocumentTemplatesResource", "AsyncDocumentTemplatesResource"]


class DocumentTemplatesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DocumentTemplatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return DocumentTemplatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DocumentTemplatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return DocumentTemplatesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        key: str,
        content: Iterable[DocumentTemplateContentRequestParam],
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DocumentTemplateCreateResponse:
        """
        Create Document template

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          content: A form field that drivers must complete when executing a route step. Defines the
              data structure and validation rules for collecting required information during
              route execution.

          name: Specify a name for the document template to be created.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/fleetify/document_templates",
            body=maybe_transform(
                {
                    "content": content,
                    "name": name,
                },
                document_template_create_params.DocumentTemplateCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, document_template_create_params.DocumentTemplateCreateParams),
            ),
            cast_to=DocumentTemplateCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DocumentTemplateRetrieveResponse:
        """
        Retrieve template by ID

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/fleetify/document_templates/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, document_template_retrieve_params.DocumentTemplateRetrieveParams),
            ),
            cast_to=DocumentTemplateRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        key: str,
        content: Iterable[DocumentTemplateContentRequestParam] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DocumentTemplateUpdateResponse:
        """
        Update a document template

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          content: An object to collect the details of form fields to be updated - data structures,
              validation rules. Please note that the details provided here will overwrite any
              existing document fields in the given template.

          name: Specify the document template name to be updated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/fleetify/document_templates/{id}",
            body=maybe_transform(
                {
                    "content": content,
                    "name": name,
                },
                document_template_update_params.DocumentTemplateUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, document_template_update_params.DocumentTemplateUpdateParams),
            ),
            cast_to=DocumentTemplateUpdateResponse,
        )

    def list(
        self,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DocumentTemplateListResponse:
        """
        Get all document templates

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/fleetify/document_templates",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, document_template_list_params.DocumentTemplateListParams),
            ),
            cast_to=DocumentTemplateListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DocumentTemplateDeleteResponse:
        """
        Delete a document template

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/fleetify/document_templates/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, document_template_delete_params.DocumentTemplateDeleteParams),
            ),
            cast_to=DocumentTemplateDeleteResponse,
        )


class AsyncDocumentTemplatesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDocumentTemplatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDocumentTemplatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDocumentTemplatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncDocumentTemplatesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        key: str,
        content: Iterable[DocumentTemplateContentRequestParam],
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DocumentTemplateCreateResponse:
        """
        Create Document template

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          content: A form field that drivers must complete when executing a route step. Defines the
              data structure and validation rules for collecting required information during
              route execution.

          name: Specify a name for the document template to be created.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/fleetify/document_templates",
            body=await async_maybe_transform(
                {
                    "content": content,
                    "name": name,
                },
                document_template_create_params.DocumentTemplateCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"key": key}, document_template_create_params.DocumentTemplateCreateParams
                ),
            ),
            cast_to=DocumentTemplateCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DocumentTemplateRetrieveResponse:
        """
        Retrieve template by ID

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/fleetify/document_templates/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"key": key}, document_template_retrieve_params.DocumentTemplateRetrieveParams
                ),
            ),
            cast_to=DocumentTemplateRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        key: str,
        content: Iterable[DocumentTemplateContentRequestParam] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DocumentTemplateUpdateResponse:
        """
        Update a document template

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          content: An object to collect the details of form fields to be updated - data structures,
              validation rules. Please note that the details provided here will overwrite any
              existing document fields in the given template.

          name: Specify the document template name to be updated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/fleetify/document_templates/{id}",
            body=await async_maybe_transform(
                {
                    "content": content,
                    "name": name,
                },
                document_template_update_params.DocumentTemplateUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"key": key}, document_template_update_params.DocumentTemplateUpdateParams
                ),
            ),
            cast_to=DocumentTemplateUpdateResponse,
        )

    async def list(
        self,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DocumentTemplateListResponse:
        """
        Get all document templates

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/fleetify/document_templates",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"key": key}, document_template_list_params.DocumentTemplateListParams
                ),
            ),
            cast_to=DocumentTemplateListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DocumentTemplateDeleteResponse:
        """
        Delete a document template

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/fleetify/document_templates/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"key": key}, document_template_delete_params.DocumentTemplateDeleteParams
                ),
            ),
            cast_to=DocumentTemplateDeleteResponse,
        )


class DocumentTemplatesResourceWithRawResponse:
    def __init__(self, document_templates: DocumentTemplatesResource) -> None:
        self._document_templates = document_templates

        self.create = to_raw_response_wrapper(
            document_templates.create,
        )
        self.retrieve = to_raw_response_wrapper(
            document_templates.retrieve,
        )
        self.update = to_raw_response_wrapper(
            document_templates.update,
        )
        self.list = to_raw_response_wrapper(
            document_templates.list,
        )
        self.delete = to_raw_response_wrapper(
            document_templates.delete,
        )


class AsyncDocumentTemplatesResourceWithRawResponse:
    def __init__(self, document_templates: AsyncDocumentTemplatesResource) -> None:
        self._document_templates = document_templates

        self.create = async_to_raw_response_wrapper(
            document_templates.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            document_templates.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            document_templates.update,
        )
        self.list = async_to_raw_response_wrapper(
            document_templates.list,
        )
        self.delete = async_to_raw_response_wrapper(
            document_templates.delete,
        )


class DocumentTemplatesResourceWithStreamingResponse:
    def __init__(self, document_templates: DocumentTemplatesResource) -> None:
        self._document_templates = document_templates

        self.create = to_streamed_response_wrapper(
            document_templates.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            document_templates.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            document_templates.update,
        )
        self.list = to_streamed_response_wrapper(
            document_templates.list,
        )
        self.delete = to_streamed_response_wrapper(
            document_templates.delete,
        )


class AsyncDocumentTemplatesResourceWithStreamingResponse:
    def __init__(self, document_templates: AsyncDocumentTemplatesResource) -> None:
        self._document_templates = document_templates

        self.create = async_to_streamed_response_wrapper(
            document_templates.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            document_templates.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            document_templates.update,
        )
        self.list = async_to_streamed_response_wrapper(
            document_templates.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            document_templates.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["NamespacedApikeyDeleteParams"]


class NamespacedApikeyDeleteParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API. Please note for the delete namespace key operation another namespace key
    cannot be used.

    The namespace created using this key can be managed using the APIs & Services >
    Credentials section of user’s
    [NextBillion Console](https://console.nextbillion.ai).
    """

    key_to_delete: Required[str]
    """Specify the key to be deleted."""

    namespace: Required[str]
    """Specify the name of the `namespace` to which the \\``key_to_delete\\`` belongs.

    Please note that a namespace key cannot be deleted using another namespace key.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["BatchCreateResponse", "Data"]


class Data(BaseModel):
    ids: Optional[List[str]] = None


class BatchCreateResponse(BaseModel):
    data: Optional[Data] = None
    """A data object containing the IDs of the geofences created."""

    status: Optional[str] = None
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages are
    returned for different errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .location_param import LocationParam

__all__ = ["VehicleParam", "Location"]


class Location(TypedDict, total=False):
    lat: float
    """Latitude of the vehicle's current location."""

    lng: float
    """Longitude of the vehicle's current location."""


class VehicleParam(TypedDict, total=False):
    id: Required[str]
    """Specify a unique ID for the vehicle."""

    location: Required[Location]
    """Specify the location coordinates where the vehicle is currently located.

    This input is mandatory for each vehicle.
    """

    attributes: object
    """Specify custom attributes for the vehicle.

    Each attribute should be created as a `key:value` pair. These attributes can be
    used in the `orders.vehicle_preferences` input to refine the search of vehicles
    for each order.

    The maximum number of key:value pairs that can be specified under `attributes`
    for a given vehicle, is limited to 30.
    """

    priority: int
    """Specify the priority for this vehicle.

    A higher value indicates a higher priority. When specified, it will override any
    priority score deduced from `vehicle_attribute_priority_mappings` for this
    vehicle. Valid values are \\[[1, 10\\]] and default is 0.
    """

    remaining_waypoints: Iterable[LocationParam]
    """
    An array of objects to collect the location coordinates of the stops remaining
    on an ongoing trip of the vehicle. The service can assign new orders to the
    vehicle if they are cost-effective. Once a new order is assigned, the vehicle
    must complete all the steps in the ongoing trip before proceeding to pickup the
    newly assigned order.

    Please note that a maximum of 10 waypoints can be specified for a given vehicle.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types import NavigationRetrieveRouteResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestNavigation:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_route(self, client: NextbillionSDK) -> None:
        navigation = client.navigation.retrieve_route(
            key="key=API_KEY",
        )
        assert_matches_type(NavigationRetrieveRouteResponse, navigation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_route_with_all_params(self, client: NextbillionSDK) -> None:
        navigation = client.navigation.retrieve_route(
            key="key=API_KEY",
            altcount=1,
            alternatives=True,
            approaches="`unrestricted`",
            avoid="toll",
            bearings="bearings=0,180;0,180",
            destination="destination=41.349302,2.136480",
            geometry="`polyline`",
            lang="lang=en",
            mode="`car`",
            origin="origin=41.349302,2.136480",
            original_shape="original_shape=sbp}_Almgp`FnLuToKmKviB{eDlcGhpFvj@qbAwoA_mA",
            original_shape_type="polyline",
            overview="`full`",
            waypoints="waypoints=41.349302,2.136480|41.349303,2.136481|41.349304,2.136482",
        )
        assert_matches_type(NavigationRetrieveRouteResponse, navigation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_route(self, client: NextbillionSDK) -> None:
        response = client.navigation.with_raw_response.retrieve_route(
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        navigation = response.parse()
        assert_matches_type(NavigationRetrieveRouteResponse, navigation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_route(self, client: NextbillionSDK) -> None:
        with client.navigation.with_streaming_response.retrieve_route(
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            navigation = response.parse()
            assert_matches_type(NavigationRetrieveRouteResponse, navigation, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncNavigation:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_route(self, async_client: AsyncNextbillionSDK) -> None:
        navigation = await async_client.navigation.retrieve_route(
            key="key=API_KEY",
        )
        assert_matches_type(NavigationRetrieveRouteResponse, navigation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_route_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        navigation = await async_client.navigation.retrieve_route(
            key="key=API_KEY",
            altcount=1,
            alternatives=True,
            approaches="`unrestricted`",
            avoid="toll",
            bearings="bearings=0,180;0,180",
            destination="destination=41.349302,2.136480",
            geometry="`polyline`",
            lang="lang=en",
            mode="`car`",
            origin="origin=41.349302,2.136480",
            original_shape="original_shape=sbp}_Almgp`FnLuToKmKviB{eDlcGhpFvj@qbAwoA_mA",
            original_shape_type="polyline",
            overview="`full`",
            waypoints="waypoints=41.349302,2.136480|41.349303,2.136481|41.349304,2.136482",
        )
        assert_matches_type(NavigationRetrieveRouteResponse, navigation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_route(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.navigation.with_raw_response.retrieve_route(
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        navigation = await response.parse()
        assert_matches_type(NavigationRetrieveRouteResponse, navigation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_route(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.navigation.with_streaming_response.retrieve_route(
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            navigation = await response.parse()
            assert_matches_type(NavigationRetrieveRouteResponse, navigation, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.skynet import namespaced_apikey_create_params, namespaced_apikey_delete_params
from ...types.skynet.namespaced_apikey_create_response import NamespacedApikeyCreateResponse
from ...types.skynet.namespaced_apikey_delete_response import NamespacedApikeyDeleteResponse

__all__ = ["NamespacedApikeysResource", "AsyncNamespacedApikeysResource"]


class NamespacedApikeysResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> NamespacedApikeysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return NamespacedApikeysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NamespacedApikeysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return NamespacedApikeysResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        key: str,
        namespace: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NamespacedApikeyCreateResponse:
        """
        Create namespace under a parent key

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          namespace: Specify a name for the `namespace`. If the namespace specified is unique then a
              new namespace along with a new key is created. Whereas if the specified
              `namespace` is not unique, a new key will be created in the existing
              `namespace`. Please note that a `namespace` cannot be created using another
              namespace key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/skynet/namespaced-apikeys",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "namespace": namespace,
                    },
                    namespaced_apikey_create_params.NamespacedApikeyCreateParams,
                ),
            ),
            cast_to=NamespacedApikeyCreateResponse,
        )

    def delete(
        self,
        *,
        key: str,
        key_to_delete: str,
        namespace: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NamespacedApikeyDeleteResponse:
        """
        Delete namespace under a parent key

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API. Please note for the delete namespace key operation another namespace key
              cannot be used.

              The namespace created using this key can be managed using the APIs & Services >
              Credentials section of user’s
              [NextBillion Console](https://console.nextbillion.ai).

          key_to_delete: Specify the key to be deleted.

          namespace: Specify the name of the `namespace` to which the \\``key_to_delete\\`` belongs.
              Please note that a namespace key cannot be deleted using another namespace key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._delete(
            "/skynet/namespaced-apikeys",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "key_to_delete": key_to_delete,
                        "namespace": namespace,
                    },
                    namespaced_apikey_delete_params.NamespacedApikeyDeleteParams,
                ),
            ),
            cast_to=NamespacedApikeyDeleteResponse,
        )


class AsyncNamespacedApikeysResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncNamespacedApikeysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNamespacedApikeysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNamespacedApikeysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncNamespacedApikeysResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        key: str,
        namespace: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NamespacedApikeyCreateResponse:
        """
        Create namespace under a parent key

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          namespace: Specify a name for the `namespace`. If the namespace specified is unique then a
              new namespace along with a new key is created. Whereas if the specified
              `namespace` is not unique, a new key will be created in the existing
              `namespace`. Please note that a `namespace` cannot be created using another
              namespace key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/skynet/namespaced-apikeys",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "namespace": namespace,
                    },
                    namespaced_apikey_create_params.NamespacedApikeyCreateParams,
                ),
            ),
            cast_to=NamespacedApikeyCreateResponse,
        )

    async def delete(
        self,
        *,
        key: str,
        key_to_delete: str,
        namespace: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NamespacedApikeyDeleteResponse:
        """
        Delete namespace under a parent key

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API. Please note for the delete namespace key operation another namespace key
              cannot be used.

              The namespace created using this key can be managed using the APIs & Services >
              Credentials section of user’s
              [NextBillion Console](https://console.nextbillion.ai).

          key_to_delete: Specify the key to be deleted.

          namespace: Specify the name of the `namespace` to which the \\``key_to_delete\\`` belongs.
              Please note that a namespace key cannot be deleted using another namespace key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._delete(
            "/skynet/namespaced-apikeys",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "key_to_delete": key_to_delete,
                        "namespace": namespace,
                    },
                    namespaced_apikey_delete_params.NamespacedApikeyDeleteParams,
                ),
            ),
            cast_to=NamespacedApikeyDeleteResponse,
        )


class NamespacedApikeysResourceWithRawResponse:
    def __init__(self, namespaced_apikeys: NamespacedApikeysResource) -> None:
        self._namespaced_apikeys = namespaced_apikeys

        self.create = to_raw_response_wrapper(
            namespaced_apikeys.create,
        )
        self.delete = to_raw_response_wrapper(
            namespaced_apikeys.delete,
        )


class AsyncNamespacedApikeysResourceWithRawResponse:
    def __init__(self, namespaced_apikeys: AsyncNamespacedApikeysResource) -> None:
        self._namespaced_apikeys = namespaced_apikeys

        self.create = async_to_raw_response_wrapper(
            namespaced_apikeys.create,
        )
        self.delete = async_to_raw_response_wrapper(
            namespaced_apikeys.delete,
        )


class NamespacedApikeysResourceWithStreamingResponse:
    def __init__(self, namespaced_apikeys: NamespacedApikeysResource) -> None:
        self._namespaced_apikeys = namespaced_apikeys

        self.create = to_streamed_response_wrapper(
            namespaced_apikeys.create,
        )
        self.delete = to_streamed_response_wrapper(
            namespaced_apikeys.delete,
        )


class AsyncNamespacedApikeysResourceWithStreamingResponse:
    def __init__(self, namespaced_apikeys: AsyncNamespacedApikeysResource) -> None:
        self._namespaced_apikeys = namespaced_apikeys

        self.create = async_to_streamed_response_wrapper(
            namespaced_apikeys.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            namespaced_apikeys.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["PolygonGeojson"]


class PolygonGeojson(BaseModel):
    coordinates: Optional[List[List[float]]] = None
    """
    An array of coordinates in the [longitude, latitude] format, representing the
    geofence boundary.
    """

    type: Optional[str] = None
    """Type of the geoJSON geometry. Will always be `Polygon`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel

__all__ = ["ConsoleSearchResponse", "Data", "DataResult"]


class DataResult(BaseModel):
    id: str
    """ID of goefence. Could be empty string."""

    name: str
    """Name of goefence. Could be empty string."""


class Data(BaseModel):
    result: List[DataResult]


class ConsoleSearchResponse(BaseModel):
    data: Data

    status: str
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages are
    returned for different errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .access import Access as Access
from .address import Address as Address
from .contacts import Contacts as Contacts
from .geofence import Geofence as Geofence
from .map_view import MapView as MapView
from .position import Position as Position
from .categories import Categories as Categories
from .post_response import PostResponse as PostResponse
from .contact_object import ContactObject as ContactObject
from .area_list_params import AreaListParams as AreaListParams
from .area_list_response import AreaListResponse as AreaListResponse
from .batch_create_params import BatchCreateParams as BatchCreateParams
from .lookup_by_id_params import LookupByIDParams as LookupByIDParams
from .rich_group_response import RichGroupResponse as RichGroupResponse
from .browse_search_params import BrowseSearchParams as BrowseSearchParams
from .geofence_list_params import GeofenceListParams as GeofenceListParams
from .batch_create_response import BatchCreateResponse as BatchCreateResponse
from .batch_retrieve_params import BatchRetrieveParams as BatchRetrieveParams
from .lookup_by_id_response import LookupByIDResponse as LookupByIDResponse
from .browse_search_response import BrowseSearchResponse as BrowseSearchResponse
from .geofence_create_params import GeofenceCreateParams as GeofenceCreateParams
from .geofence_delete_params import GeofenceDeleteParams as GeofenceDeleteParams
from .geofence_list_response import GeofenceListResponse as GeofenceListResponse
from .geofence_update_params import GeofenceUpdateParams as GeofenceUpdateParams
from .batch_retrieve_response import BatchRetrieveResponse as BatchRetrieveResponse
from .geocode_retrieve_params import GeocodeRetrieveParams as GeocodeRetrieveParams
from .restriction_list_params import RestrictionListParams as RestrictionListParams
from .skynet_subscribe_params import SkynetSubscribeParams as SkynetSubscribeParams
from .discover_retrieve_params import DiscoverRetrieveParams as DiscoverRetrieveParams
from .geofence_contains_params import GeofenceContainsParams as GeofenceContainsParams
from .geofence_create_response import GeofenceCreateResponse as GeofenceCreateResponse
from .geofence_retrieve_params import GeofenceRetrieveParams as GeofenceRetrieveParams
from .isochrone_compute_params import IsochroneComputeParams as IsochroneComputeParams
from .snap_to_road_snap_params import SnapToRoadSnapParams as SnapToRoadSnapParams
from .geocode_retrieve_response import GeocodeRetrieveResponse as GeocodeRetrieveResponse
from .restriction_create_params import RestrictionCreateParams as RestrictionCreateParams
from .restriction_delete_params import RestrictionDeleteParams as RestrictionDeleteParams
from .restriction_list_response import RestrictionListResponse as RestrictionListResponse
from .restriction_update_params import RestrictionUpdateParams as RestrictionUpdateParams
from .skynet_subscribe_response import SkynetSubscribeResponse as SkynetSubscribeResponse
from .autosuggest_suggest_params import AutosuggestSuggestParams as AutosuggestSuggestParams
from .discover_retrieve_response import DiscoverRetrieveResponse as DiscoverRetrieveResponse
from .geofence_contains_response import GeofenceContainsResponse as GeofenceContainsResponse
from .geofence_retrieve_response import GeofenceRetrieveResponse as GeofenceRetrieveResponse
from .isochrone_compute_response import IsochroneComputeResponse as IsochroneComputeResponse
from .multigeocode_search_params import MultigeocodeSearchParams as MultigeocodeSearchParams
from .revgeocode_retrieve_params import RevgeocodeRetrieveParams as RevgeocodeRetrieveParams
from .route_report_create_params import RouteReportCreateParams as RouteReportCreateParams
from .snap_to_road_snap_response import SnapToRoadSnapResponse as SnapToRoadSnapResponse
from .autocomplete_suggest_params import AutocompleteSuggestParams as AutocompleteSuggestParams
from .geocode_batch_create_params import GeocodeBatchCreateParams as GeocodeBatchCreateParams
from .optimization_compute_params import OptimizationComputeParams as OptimizationComputeParams
from .restriction_delete_response import RestrictionDeleteResponse as RestrictionDeleteResponse
from .restriction_retrieve_params import RestrictionRetrieveParams as RestrictionRetrieveParams
from .autosuggest_suggest_response import AutosuggestSuggestResponse as AutosuggestSuggestResponse
from .geofence_entity_create_param import GeofenceEntityCreateParam as GeofenceEntityCreateParam
from .multigeocode_search_response import MultigeocodeSearchResponse as MultigeocodeSearchResponse
from .restriction_set_state_params import RestrictionSetStateParams as RestrictionSetStateParams
from .revgeocode_retrieve_response import RevgeocodeRetrieveResponse as RevgeocodeRetrieveResponse
from .route_report_create_response import RouteReportCreateResponse as RouteReportCreateResponse
from .autocomplete_suggest_response import AutocompleteSuggestResponse as AutocompleteSuggestResponse
from .geocode_batch_create_response import GeocodeBatchCreateResponse as GeocodeBatchCreateResponse
from .optimization_compute_response import OptimizationComputeResponse as OptimizationComputeResponse
from .restrictions_item_list_params import RestrictionsItemListParams as RestrictionsItemListParams
from .direction_compute_route_params import DirectionComputeRouteParams as DirectionComputeRouteParams
from .optimization_re_optimize_params import OptimizationReOptimizeParams as OptimizationReOptimizeParams
from .restriction_list_by_bbox_params import RestrictionListByBboxParams as RestrictionListByBboxParams
from .restrictions_item_list_response import RestrictionsItemListResponse as RestrictionsItemListResponse
from .direction_compute_route_response import DirectionComputeRouteResponse as DirectionComputeRouteResponse
from .navigation_retrieve_route_params import NavigationRetrieveRouteParams as NavigationRetrieveRouteParams
from .mdm_create_distance_matrix_params import MdmCreateDistanceMatrixParams as MdmCreateDistanceMatrixParams
from .restriction_list_by_bbox_response import RestrictionListByBboxResponse as RestrictionListByBboxResponse
from .geocode_structured_retrieve_params import GeocodeStructuredRetrieveParams as GeocodeStructuredRetrieveParams
from .navigation_retrieve_route_response import NavigationRetrieveRouteResponse as NavigationRetrieveRouteResponse
from .mdm_create_distance_matrix_response import MdmCreateDistanceMatrixResponse as MdmCreateDistanceMatrixResponse
from .geocode_structured_retrieve_response import GeocodeStructuredRetrieveResponse as GeocodeStructuredRetrieveResponse
from .mdm_get_distance_matrix_status_params import MdmGetDistanceMatrixStatusParams as MdmGetDistanceMatrixStatusParams
from .postalcode_retrieve_coordinates_params import (
    PostalcodeRetrieveCoordinatesParams as PostalcodeRetrieveCoordinatesParams,
)
from .mdm_get_distance_matrix_status_response import (
    MdmGetDistanceMatrixStatusResponse as MdmGetDistanceMatrixStatusResponse,
)
from .postalcode_retrieve_coordinates_response import (
    PostalcodeRetrieveCoordinatesResponse as PostalcodeRetrieveCoordinatesResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .steps import (
    StepsResource,
    AsyncStepsResource,
    StepsResourceWithRawResponse,
    AsyncStepsResourceWithRawResponse,
    StepsResourceWithStreamingResponse,
    AsyncStepsResourceWithStreamingResponse,
)
from .routes import (
    RoutesResource,
    AsyncRoutesResource,
    RoutesResourceWithRawResponse,
    AsyncRoutesResourceWithRawResponse,
    RoutesResourceWithStreamingResponse,
    AsyncRoutesResourceWithStreamingResponse,
)

__all__ = [
    "StepsResource",
    "AsyncStepsResource",
    "StepsResourceWithRawResponse",
    "AsyncStepsResourceWithRawResponse",
    "StepsResourceWithStreamingResponse",
    "AsyncStepsResourceWithStreamingResponse",
    "RoutesResource",
    "AsyncRoutesResource",
    "RoutesResourceWithRawResponse",
    "AsyncRoutesResourceWithRawResponse",
    "RoutesResourceWithStreamingResponse",
    "AsyncRoutesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["OptimizationComputeParams"]


class OptimizationComputeParams(TypedDict, total=False):
    coordinates: Required[str]
    """This is a pipe-separated list of coordinates.

    Minimum 3 pairs of coordinates and Maximum 12 pairs of coordinates are allowed.
    """

    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    approaches: Literal["`unrestricted`", "`curb`"]
    """
    A semicolon-separated list indicating the side of the road from which to
    approach waypoints in a requested route. If provided, the number of `approaches`
    must be the same as the number of `coordinates`. However, you can skip a
    coordinate and show its position in the list with the `;` separator.
    """

    destination: Literal["`any`", "`last`"]
    """Specify the destination coordinate of the returned route.

    If the input is `last`, the last coordinate will be the destination.
    """

    geometries: Literal["`polyline`", "`polyline6`", "`geojson`"]
    """Sets the output format of the route geometry in the response.

    On providing `polyline` and `polyline6` as input, respective encoded geometry is
    returned. However, when `geojson` is provided as the input value, `polyline`
    encoded geometry is returned in the response along with a geojson details of the
    route.
    """

    mode: Literal["`car`", "`truck`"]
    """Set which driving mode the service should use to determine a route.

    For example, if you use "car", the API will return a route that a car can take.
    Using "truck" will return a route a truck can use, taking into account
    appropriate truck routing restrictions.

    When "mode=truck", following are the default dimensions that are used:

    \\-- truck_height = 214 centimeters

    \\-- truck_width = 183 centimeters

    \\-- truck_length = 519 centimeters

    \\-- truck_weight = 5000 kg

    Please use the Directions Flexible version if you want to use custom truck
    dimensions.

    Note: Only the "car" profile is enabled by default. Please note that customized
    profiles (including "truck") might not be available for all regions. Please
    contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
    representative or reach out at
    [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
    additional profiles.
    """

    roundtrip: bool
    """Indicates whether the returned route is a roundtrip."""

    source: Literal["`any`", "`first`"]
    """The coordinate at which to start the returned route.

    If this is not configured, the return route’s destination will be the first
    coordinate.
    """

    with_geometry: bool
    """Indicates whether the return geometry should be computed or not."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["AutosuggestSuggestParams"]


class AutosuggestSuggestParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    q: Required[str]
    """Specify the free-text search query.

    Please note that whitespace, urls, email addresses, or other out-of-scope
    queries will yield no results.
    """

    at: str
    """Specify the center of the search context expressed as coordinates.

    Please note that one of "at", "in=circle" or "in=bbox" should be provided for
    relevant results.
    """

    in_: Annotated[str, PropertyInfo(alias="in")]
    """Search within a geographic area.

    This is a hard filter. Results will be returned if they are located within the
    specified area.

    A geographic area can be

    - a country (or multiple countries), provided as comma-separated
      [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country
      codes

      The country codes are to be provided in all uppercase.

      Format: `countryCode:{countryCode}[,{countryCode}]`

    - a circular area, provided as latitude, longitude, and radius (an integer with
      meters as unit)

      Format: `circle:{latitude},{longitude};r={radius}`

    - a bounding box, provided as _west longitude_, _south latitude_, _east
      longitude_, _north latitude_

      Format:
      `bbox:{west longitude},{south latitude},{east longitude},{north latitude}`

    Please provide one of 'at', 'in=circle' or 'in=bbox' input for a relevant
    result.
    """

    lang: str
    """
    Select the language to be used for result rendering from a list of
    [BCP 47](https://en.wikipedia.org/wiki/IETF_language_tag) compliant language
    codes.
    """

    limit: int
    """Sets the maximum number of results to be returned."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = [
    "DirectionComputeRouteResponse",
    "Route",
    "RouteEndLocation",
    "RouteGeojson",
    "RouteGeojsonGeometry",
    "RouteLeg",
    "RouteLegDistance",
    "RouteLegDuration",
    "RouteLegEndLocation",
    "RouteLegStartLocation",
    "RouteLegStep",
    "RouteLegStepDistance",
    "RouteLegStepDuration",
    "RouteLegStepEndLocation",
    "RouteLegStepGeojson",
    "RouteLegStepGeojsonGeometry",
    "RouteLegStepManeuver",
    "RouteLegStepManeuverCoordinate",
    "RouteLegStepStartLocation",
    "RouteStartLocation",
]


class RouteEndLocation(BaseModel):
    latitude: Optional[float] = None
    """latitude of the `start_location`."""

    longitude: Optional[float] = None
    """longitude of the `start_location`."""


class RouteGeojsonGeometry(BaseModel):
    coordinates: Optional[List[float]] = None
    """
    An array of coordinates in the [longitude, latitude] format, representing the
    route geometry.
    """

    type: Optional[str] = None
    """Type of the geoJSON geometry."""


class RouteGeojson(BaseModel):
    geometry: Optional[RouteGeojsonGeometry] = None
    """An object with details of the geoJSON geometry of the route."""

    properties: Optional[str] = None
    """Property associated with the geoJSON shape."""

    type: Optional[str] = None
    """Type of the geoJSON object."""


class RouteLegDistance(BaseModel):
    value: Optional[float] = None


class RouteLegDuration(BaseModel):
    value: Optional[float] = None


class RouteLegEndLocation(BaseModel):
    latitude: Optional[float] = None
    """Latitude of the `end_location` of the `leg`."""

    longitude: Optional[float] = None
    """Longitude of the `end_location` of the `leg`."""


class RouteLegStartLocation(BaseModel):
    latitude: Optional[float] = None
    """Latitude of the `start_location` of the `leg`."""

    longitude: Optional[float] = None
    """Longitude of the `start_location` of the `leg`."""


class RouteLegStepDistance(BaseModel):
    value: Optional[float] = None


class RouteLegStepDuration(BaseModel):
    value: Optional[float] = None


class RouteLegStepEndLocation(BaseModel):
    latitude: Optional[float] = None
    """Latitude of the `end_location` of the `step`."""

    longitude: Optional[float] = None
    """Longitude of the `end_location` of the `step`."""


class RouteLegStepGeojsonGeometry(BaseModel):
    coordinates: Optional[List[float]] = None
    """
    An array of coordinates in the [longitude, latitude] format, representing the
    `step` geometry.
    """

    type: Optional[str] = None
    """Type of the geoJSON geometry."""


class RouteLegStepGeojson(BaseModel):
    geometry: Optional[RouteLegStepGeojsonGeometry] = None
    """An object with details of the geoJSON geometry of the `step`."""

    properties: Optional[str] = None
    """Property associated with the geoJSON shape."""

    type: Optional[str] = None
    """Type of the geoJSON object."""


class RouteLegStepManeuverCoordinate(BaseModel):
    latitude: Optional[float] = None
    """Latitude of the `maneuver` location."""

    longitude: Optional[float] = None
    """Longitude of the `maneuver` location."""


class RouteLegStepManeuver(BaseModel):
    bearing_after: Optional[int] = None
    """
    The clockwise angle from true north to the direction of travel immediately after
    the `maneuver`. Range of values is between 0-359.
    """

    bearing_before: Optional[int] = None
    """
    The clockwise angle from true north to the direction of travel immediately
    before the `maneuver`. Range of values is between 0-359.
    """

    coordinate: Optional[RouteLegStepManeuverCoordinate] = None
    """A coordinate pair describing the location of the `maneuver`."""

    maneuver_type: Optional[str] = None
    """A string indicating the type of `maneuver`."""

    modifier: Optional[str] = None
    """Modifier associated with `maneuver_type`."""


class RouteLegStepStartLocation(BaseModel):
    latitude: Optional[float] = None
    """Latitude of the `start_location` of the `step`."""

    longitude: Optional[float] = None
    """Longitude of the `start_location` of the `step`."""


class RouteLegStep(BaseModel):
    distance: Optional[RouteLegStepDistance] = None
    """An object containing step distance value, in meters."""

    duration: Optional[RouteLegStepDuration] = None
    """An object containing step duration value, in seconds."""

    end_location: Optional[RouteLegStepEndLocation] = None
    """Location coordinates of the point where the `step` ends."""

    geojson: Optional[RouteLegStepGeojson] = None
    """
    An object with geoJSON details of the `step`.This object is returned when the
    `geometry` field is set to `geojson` in the input request, otherwise it is not
    present in the response. The contents of this object follow the
    [geoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946).
    """

    geometry: Optional[str] = None
    """Encoded geometry of the `step` in the selected format."""

    maneuver: Optional[RouteLegStepManeuver] = None
    """An object with maneuver details for the `step`."""

    start_location: Optional[RouteLegStepStartLocation] = None
    """Location coordinates of the point where the `step` starts."""


class RouteLeg(BaseModel):
    distance: Optional[RouteLegDistance] = None
    """An object containing leg distance value, in meters."""

    duration: Optional[RouteLegDuration] = None
    """An object containing leg duration value, in seconds."""

    end_location: Optional[RouteLegEndLocation] = None
    """Location coordinates of the point where the leg ends.

    Returned only when `steps` is true in the input request.
    """

    start_location: Optional[RouteLegStartLocation] = None
    """Location coordinates of the point where the leg starts.

    Returned only when `steps` is true in the input request.
    """

    steps: Optional[List[RouteLegStep]] = None
    """An array of objects with details of each step of the `legs`.

    Returned only when `steps` is `true` in the input request. An empty array is
    returned when `steps` is `false` in the input request.
    """


class RouteStartLocation(BaseModel):
    latitude: Optional[float] = None
    """Latitude of the `start_location`."""

    longitude: Optional[float] = None
    """Longitude of the `start_location`."""


class Route(BaseModel):
    distance: Optional[float] = None
    """The distance, in meters, for the complete trip."""

    duration: Optional[float] = None
    """The duration, in seconds, of the complete trip."""

    end_location: Optional[RouteEndLocation] = None
    """Location coordinates of the point where the route ends.

    It is the same as the `destination` in the input request. Returned only when
    `steps` is true in the input request.
    """

    geojson: Optional[RouteGeojson] = None
    """An object with geoJSON details of the route.

    This object is returned when the `geometry` field is set to `geojson` in the
    input request, otherwise it is not present in the response. The contents of this
    object follow the
    [geoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946).
    """

    geometry: Optional[str] = None
    """
    Encoded geometry of the returned route in the selected format and specified
    `overview` verbosity. This parameter is configured in the input request.
    """

    legs: Optional[List[RouteLeg]] = None
    """An array of objects returning the details about each `leg` of the route.

    `waypoints` split the route into legs.
    """

    start_location: Optional[RouteStartLocation] = None
    """Location coordinates of the point where the route starts.

    It is the same as the `origin` in the input request. Returned only when `steps`
    is true in the input request.
    """


class DirectionComputeRouteResponse(BaseModel):
    msg: Optional[str] = None
    """Displays the error message in case of a failed request or operation.

    Please note that this parameter is not returned in the response in case of a
    successful request.
    """

    route: Optional[Route] = None
    """An object containing details about the returned route.

    Will contain multiple objects if more than one routes are present in the
    response.
    """

    status: Optional[str] = None
    """A string indicating the state of the response.

    On normal responses, the value will be `Ok`. Indicative HTTP error codes are
    returned for different errors. See the [API Errors Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from .place_item import PlaceItem

__all__ = ["PlaceRetrieveResponse", "DataSorce"]


class DataSorce(BaseModel):
    ref_id: Optional[str] = FieldInfo(alias="refId", default=None)
    """
    This parameter represents the unique reference ID associated with the data
    source.
    """

    source: Optional[str] = None
    """
    This parameter represents the current dataset source of the information returned
    in the result.
    """

    status: Optional[Literal["enable", "disable"]] = None
    """
    This parameter indicates if a place is currently discoverable by search API or
    not.
    """


class PlaceRetrieveResponse(BaseModel):
    data_sorce: Optional[DataSorce] = FieldInfo(alias="dataSorce", default=None)
    """
    It displays the information about the current source and current status of the
    place. Use the “Update Place” method to change these values, as needed.
    """

    doc_id: Optional[str] = FieldInfo(alias="docId", default=None)
    """The unique NextBillion ID for the result item."""

    place: Optional[List[PlaceItem]] = None
    """
    This parameter represents the place details, including geographical information,
    address and other related information.
    """

    score: Optional[int] = None
    """It returns the system calculated weighted score of the place.

    It depends on how ‘richly’ the place was defined at the time of creation. In
    order to modify the score, use “Update Place” method and update information for
    parameters which are not set currently. As an alternative, you can directly
    update the `score` to a custom value.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SkynetSubscribeParams", "Params"]


class SkynetSubscribeParams(TypedDict, total=False):
    action: Required[Literal["`TRIP_SUBSCRIBE`", "`TRIP_UNSUBSCRIBE`", "`HEARTBEAT`"]]
    """Specify the behavior that needs to be achieved for the subscription.

    Following values are accepted:

    - TRIP_SUBSCRIBE: Enable a trip subscription.
    - TRIP_UNSUBSCRIBE: Unsubscribe from a trip
    - HEARTBEAT: Enable heartbeat mechanism for a web-socket connection. The action
      message need to be sent at a frequency higher than every 5 mins to keep the
      connection alive. Alternatively, users can chose to respond to the ping frame
      sent by web socket server to keep the connection alive. Refer to
      [connection details](https://188--nbai-docs-stg.netlify.app/docs/tracking/api/live-tracking-api#connect-to-web-socket-server)
      for more details.
    """

    id: str
    """Specify a custom ID for the subscription.

    It can be used to reference a given subscription in the downstream applications
    / systems.
    """

    params: Params


class Params(TypedDict, total=False):
    id: Required[str]
    """Specify the ID of an active trip that needs to be subscribed.

    The ID of a trip is returned in the response when _Start A Trip_ request is
    acknowledged.

    This attribute is mandatory when `action` is set to either "TRIP_SUBSCRIBE" or
    "TRIP_UNSUBSCRIBE"
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["DocumentTemplateContentResponse", "Meta", "MetaOption", "Validation"]


class MetaOption(BaseModel):
    label: Optional[str] = None
    """Returns the label for the option."""

    value: Optional[str] = None
    """Returns the value associated with the option.

    This value gets submitted when the option is checked in the Driver app.
    """


class Meta(BaseModel):
    options: Optional[List[MetaOption]] = None
    """
    An array of objects returning the options for `multi_choices` or `single_choice`
    type document field. Each object represents one configured option.
    """


class Validation(BaseModel):
    max: Optional[int] = None
    """
    Returns the maximum allowed value for `number` type document item, as specified
    at the time of configuring the field. This parameter is not present in the
    response if it was not provided in the input.
    """

    max_items: Optional[str] = None
    """
    Returns the maximum number of items required for `multi_choices`, `photos` type
    document items. This parameter will not be present in the response if it was not
    provided in the input.
    """

    min: Optional[int] = None
    """
    Returns the minimum allowed value for `number` type document item, as specified
    at the time of configuring the field. This parameter is not present in the
    response if it was not provided in the input.
    """

    min_items: Optional[str] = None
    """
    Returns the minimum number of items required for `multi_choices`, `photos` type
    document items. This parameter will not be present in the response if it was not
    provided in the input.
    """


class DocumentTemplateContentResponse(BaseModel):
    label: Optional[str] = None
    """Returns the label of the document field."""

    meta: Optional[Meta] = None
    """
    Returns the options configured for `single_choice` or `multi_choices` type
    document items.
    """

    name: Optional[str] = None
    """Returns the name of the document field."""

    required: Optional[bool] = None
    """Indicates if the document field is mandatory or not."""

    type: Optional[str] = None
    """Returns the data type of the document field.

    It will always belong to one of `string`, `number`, `date_time`, `photos`,
    `multi_choices`, `signature`, `barcode`, and `single_choice.`
    """

    validation: Optional[Validation] = None
    """
    Returns the validation rules for `number` , `multi_choices` , and `photos`
    document field types.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

from .meta_data_param import MetaDataParam

__all__ = ["AssetUpdateParams"]


class AssetUpdateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    cluster: Literal["america"]
    """the cluster of the region you want to use"""

    attributes: object
    """Use this param to update the `attributes` of an asset in `key`:`value` format.

    Users can maintain any useful information or context about the assets by
    utilising this parameter.

    Please be careful when using this parameter while updating an `asset` as the new
    `attributes` object provided will completely overwrite the old `attributes`
    object. Use the _Update Asset Attributes_ method to add new or modify existing
    attributes.

    Another point to note is that the overall size of the `attributes` object cannot
    exceed 65kb and the maximum number of `key`:`value` pairs that can be added to
    this object is 100.
    """

    description: str
    """Use this param to update the `description` of an `asset`."""

    meta_data: MetaDataParam
    """Any valid json object data.

    Can be used to save customized data. Max size is 65kb.
    """

    name: str
    """Use this param to update the `name` of an `asset`.

    Users can assign meaningful custom names to their assets.
    """

    tags: List[str]
    """
    **This parameter will be deprecated soon! Please use the `attributes` parameter
    to add labels or markers for the asset.**

    Use this param to update the `tags` of an `asset`. `tags` can be used to filter
    `asset` in _Get Asset List_, **Search** and **Monitor** queries.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["MultigeocodeSearchParams", "At"]


class MultigeocodeSearchParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    at: Required[At]
    """Specify the center of the search context expressed as coordinates."""

    query: Required[str]
    """A free-form, complete or incomplete string to be searched.

    It allows searching for places using keywords or names.
    """

    city: str
    """Specifies the primary city of the place."""

    country: str
    """
    Country of the search context provided as comma-separated
    [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country
    codes.
    Note: Country codes should be provided in uppercase.
    """

    district: str
    """Specifies the district of the search place."""

    limit: int
    """Sets the maximum number of results to be returned."""

    radius: str
    """
    Filters the results to places within the specified radius from the 'at'
    location.

    Note: Supports 'meter' (m) and 'kilometer' (km) units. If no radius is given,
    the search method returns as many results as specified in `limit`.
    """

    state: str
    """Specifies the state of the search place."""

    street: str
    """Specifies the street name of the search place."""

    sub_district: Annotated[str, PropertyInfo(alias="subDistrict")]
    """Specifies the subDistrict of the search place."""


class At(TypedDict, total=False):
    lat: Required[float]
    """Latitude coordinate of the location"""

    lng: Required[float]
    """Longitude coordinate of the location."""
